import { __decorate, __param } from 'tslib';
import { ɵɵdefineInjectable, Injectable, Component, EventEmitter, ElementRef, Inject, Output, Input, Directive, NgModule, Pipe, RendererFactory2, ɵɵinject } from '@angular/core';
import { CommonModule } from '@angular/common';
import { Observable, of, BehaviorSubject, forkJoin } from 'rxjs';
import { AbstractControl } from '@angular/forms';
import * as jalaliMomentImported from 'jalali-moment';
import * as momentImported from 'moment';
import swal from 'sweetalert2';
import { DomSanitizer, Title } from '@angular/platform-browser';
import { take, map } from 'rxjs/operators';
import { Router } from '@angular/router';
import { TranslateService } from '@ngx-translate/core';

let UtilService = class UtilService {
    constructor() { }
};
UtilService.ngInjectableDef = ɵɵdefineInjectable({ factory: function UtilService_Factory() { return new UtilService(); }, token: UtilService, providedIn: "root" });
UtilService = __decorate([
    Injectable({
        providedIn: 'root'
    })
], UtilService);

let UtilComponent = class UtilComponent {
    constructor() { }
    ngOnInit() {
    }
};
UtilComponent = __decorate([
    Component({
        selector: 'lib-util',
        template: `
    <p>
      util works!
    </p>
  `
    })
], UtilComponent);

function isNullOrUndefinedOrBlankString(value) {
    return value === undefined || value === null || value === '';
}
function isNullOrUndefined(value) {
    return value === undefined || value === null;
}
function isUndefined(value) {
    return value === undefined;
}
function isBlankString(value) {
    return value === '';
}
function isFunction(value) {
    return typeof (value) === 'function';
}
function objAndSameType(obj1, obj2) {
    return isObject(obj1) && isObject(obj2)
        && Object.getPrototypeOf(obj1) === Object.getPrototypeOf(obj2)
        && !(Array.isArray(obj1) || Array.isArray(obj2));
}
function isObject(x) {
    return x != null && typeof x === 'object';
}
function clone(value) {
    if (!isObject(value) || value instanceof RegExp || value instanceof Observable) {
        return value;
    }
    if (value instanceof AbstractControl) {
        return null;
    }
    if (Object.prototype.toString.call(value) === '[object Date]') {
        return new Date(value.getTime());
    }
    if (Array.isArray(value)) {
        return value.slice(0).map(v => clone(v));
    }
    value = Object.assign({}, value);
    Object.keys(value).forEach(k => value[k] = clone(value[k]));
    return value;
}
function copyTextToClipboard(text) {
    const textArea = document.createElement('textarea');
    //
    // *** This styling is an extra step which is likely not required. ***
    //
    // Why is it here? To ensure:
    // 1. the element is able to have focus and selection.
    // 2. if element was to flash render it has minimal visual impact.
    // 3. less flakyness with selection and copying which **might** occur if
    //    the textarea element is not visible.
    //
    // The likelihood is the element won't even render, not even a
    // flash, so some of these are just precautions. However in
    // Internet Explorer the element is visible whilst the popup
    // box asking the user for permission for the web page to
    // copy to the clipboard.
    //
    // Place in top-left corner of screen regardless of scroll position.
    textArea.style.position = 'fixed';
    textArea.style.top = '0';
    textArea.style.left = '0';
    // Ensure it has a small width and height. Setting to 1px / 1em
    // doesn't work as this gives a negative w/h on some browsers.
    textArea.style.width = '2em';
    textArea.style.height = '2em';
    // We don't need padding, reducing the size if it does flash render.
    textArea.style.padding = '0';
    // Clean up any borders.
    textArea.style.border = 'none';
    textArea.style.outline = 'none';
    textArea.style.boxShadow = 'none';
    // Avoid flash of white box if rendered for any reason.
    textArea.style.background = 'transparent';
    textArea.value = text;
    document.body.appendChild(textArea);
    textArea.focus();
    textArea.select();
    let commandResult = false;
    try {
        commandResult = document.execCommand('copy');
        // const msg = commandResult ? 'successful' : 'unsuccessful';
        // console.log('Copying text command was ' + msg);
    }
    catch (err) {
        console.error('Oops, unable to copy');
    }
    document.body.removeChild(textArea);
    return commandResult;
}

let AreYouSureDirective = class AreYouSureDirective {
    constructor(element) {
        this.element = element;
        this.then = new EventEmitter();
        this.else = new EventEmitter();
    }
    ngOnInit() {
        const directive = this;
        this.element.nativeElement.onclick = function () {
            const result = confirm('آیا مطمئن هستید؟');
            if (result) {
                directive.then.emit(true);
            }
            else {
                directive.else.emit(true);
            }
        };
    }
    ngOnChanges(changes) {
        if (changes.hasOwnProperty('message')) {
            if (!isNullOrUndefined(this.message))
                this.selfMessage = this.message;
        }
    }
};
AreYouSureDirective.ctorParameters = () => [
    { type: ElementRef, decorators: [{ type: Inject, args: [ElementRef,] }] }
];
__decorate([
    Output()
], AreYouSureDirective.prototype, "then", void 0);
__decorate([
    Output()
], AreYouSureDirective.prototype, "else", void 0);
__decorate([
    Input()
], AreYouSureDirective.prototype, "message", void 0);
AreYouSureDirective = __decorate([
    Directive({
        selector: '[nbdAreYouSure]'
    }),
    __param(0, Inject(ElementRef))
], AreYouSureDirective);

let NbDirectivesModule = class NbDirectivesModule {
};
NbDirectivesModule = __decorate([
    NgModule({
        imports: [
            CommonModule
        ],
        declarations: [AreYouSureDirective],
        exports: [
            AreYouSureDirective
        ]
    })
], NbDirectivesModule);

/**
 * @author Jafar Amini in March 2018.
 */
class Common {
    static create() {
        return new Common();
    }
    static checkIsNullOrUndefined(value) {
        return isNullOrUndefined(value);
    }
    // تبدیل اعداد انگلیسی به فارسی
    static En2Fa(value) {
        const englishNumbers = ['۱', '۲', '۳', '۴', '۵', '۶', '۷', '۸', '۹', '۰'];
        const persianNumbers = ['1', '2', '3', '4', '5', '6', '7', '8', '9', '0'];
        if (isNullOrUndefined(value))
            return value;
        let myvalue = value.toString();
        if (!isNullOrUndefined(value)) {
            for (let i = 0, numbersLen = persianNumbers.length; i < numbersLen; i++) {
                myvalue = myvalue.replace(new RegExp(persianNumbers[i], 'g'), englishNumbers[i]);
            }
        }
        return myvalue;
    }
    // تبدیل اعداد فارسی به انگلیسی
    static Fa2En(value) {
        const englishNumbers = ['1', '2', '3', '4', '5', '6', '7', '8', '9', '0'];
        const persianNumbers = ['۱', '۲', '۳', '۴', '۵', '۶', '۷', '۸', '۹', '۰'];
        if (isNullOrUndefined(value))
            return value;
        let myvalue = value.toString();
        if (!isNullOrUndefined(value)) {
            for (let i = 0, numbersLen = persianNumbers.length; i < numbersLen; i++) {
                myvalue = myvalue.replace(new RegExp(persianNumbers[i], 'g'), englishNumbers[i]);
            }
        }
        return myvalue;
    }
    uuidv4() {
        return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'
            .replace(/[xy]/g, (c) => {
            let r = Math.random() * 16 | 0, v = c == 'x' ? r : (r & 0x3 | 0x8);
            return v.toString(16);
        });
    }
}

/**
 * @author Jafar Amini in March 2018.
 */
const jalaliMoment = jalaliMomentImported;
const moment = momentImported;
class Moment {
    // *************************************
    static isoToJalaliFormat(date, format) {
        return jalaliMoment(Date.parse(date)).format(format);
    }
    static isoToJDay(date) {
        if (date) {
            return Moment.isoToJalaliFormat(date, 'jYYYY/jMM/jDD');
        }
    }
    static isoToJMin(date) {
        if (date) {
            return Moment.isoToJalaliFormat(date, 'HH:mm - jYYYY/jMM/jDD');
        }
    }
    static isoToJSec(date) {
        if (date) {
            return Moment.isoToJalaliFormat(date, 'HH:mm:ss - jYYYY/jMM/jDD');
        }
    }
    // *************************************
    static isoToGregorianFormat(date, format) {
        return moment(date).format(format);
    }
    static isoToGDay(date) {
        if (date) {
            return Moment.isoToGregorianFormat(date, 'YYYY/MM/DD');
        }
    }
    static isoToGMin(date) {
        if (date) {
            return Moment.isoToGregorianFormat(date, 'HH:mm - YYYY/MM/DD');
        }
    }
    static isoToGSec(date) {
        if (date) {
            return Moment.isoToGregorianFormat(date, 'HH:mm:ss - YYYY/MM/DD');
        }
    }
    // *************************************
    static getJaliliDateFromIsoOrFull(date) {
        if (date) {
            return jalaliMoment(date).format('HH:mm - jYYYY/jMM/jDD');
        }
    }
    static getGregorianDateFromIsoOrFull(date) {
        if (date) {
            return jalaliMoment(date).format('HH:mm - YYYY/MM/DD');
        }
    }
    static convertJaliliToGregorian(jDate) {
        return jalaliMoment(jalaliMoment(Common.Fa2En(jDate), 'jYYYY/jMM/jDD').toISOString()).format('YYYY/MM/DD');
    }
    static convertJaliliToGregorianWithTime(jDate) {
        return jalaliMoment(jalaliMoment(Common.Fa2En(jDate), 'HH:mm - jYYYY/jMM/jDD').toISOString()).format('HH:mm - YYYY/MM/DD');
    }
    static convertGregorianToJalili(date) {
        return jalaliMoment(jalaliMoment(Common.Fa2En(date), 'YYYY/MM/DD').toISOString()).format('YYYY/MM/DD');
    }
    static convertGregorianToJaliliWithTime(date) {
        return jalaliMoment(jalaliMoment(Common.Fa2En(date), 'HH:mm - YYYY/MM/DD').toISOString())
            .format('HH:mm - YYYY/MM/DD');
    }
    static convertJaliliToIsoDate(jDate) {
        return jalaliMoment(Common.Fa2En(jDate), 'jYYYY/jMM/jDD').toISOString();
    }
    static convertGregorianToIsoDate(jDate) {
        return jalaliMoment(Common.Fa2En(jDate), 'YYYY/MM/DD').toISOString();
    }
}

let IsoToJDayPipe = class IsoToJDayPipe {
    transform(value) {
        return Moment.isoToJDay(value);
    }
};
IsoToJDayPipe = __decorate([
    Pipe({
        name: 'isoToJDay'
    })
], IsoToJDayPipe);

let IsoToJMinPipe = class IsoToJMinPipe {
    transform(value) {
        return Moment.isoToJMin(value);
    }
};
IsoToJMinPipe = __decorate([
    Pipe({
        name: 'isoToJMin'
    })
], IsoToJMinPipe);

let IsoToJSecPipe = class IsoToJSecPipe {
    transform(value) {
        return Moment.isoToJSec(value);
    }
};
IsoToJSecPipe = __decorate([
    Pipe({
        name: 'isoToJSec'
    })
], IsoToJSecPipe);

let IsoToJalaliPipe = class IsoToJalaliPipe {
    transform(value, type) {
        switch (type) {
            case 'day':
                return Moment.isoToJDay(value);
            case 'min':
                return Moment.isoToJMin(value);
            case 'sec':
                return Moment.isoToJSec(value);
        }
    }
};
IsoToJalaliPipe = __decorate([
    Pipe({
        name: 'isoToJalali'
    })
], IsoToJalaliPipe);

let PipeMomentModule = class PipeMomentModule {
};
PipeMomentModule = __decorate([
    NgModule({
        imports: [
            CommonModule
        ],
        declarations: [IsoToJDayPipe, IsoToJMinPipe, IsoToJSecPipe, IsoToJalaliPipe],
        exports: [
            IsoToJDayPipe,
            IsoToJMinPipe,
            IsoToJSecPipe,
            IsoToJalaliPipe
        ]
    })
], PipeMomentModule);

let NbpipePipe = class NbpipePipe {
    transform(value, mappingFunction) {
        return mappingFunction(value);
    }
};
NbpipePipe = __decorate([
    Pipe({
        name: 'nbpipe'
    })
], NbpipePipe);

/**
 * @author Jafar Amini in May 2018.
 */
class PairKeyValue {
    constructor(key, value) {
        this.key = key;
        this.value = value;
    }
    static getValue(key, pairKeyValueList) {
        if (pairKeyValueList) {
            for (let pair of pairKeyValueList) {
                if (pair.key === key) {
                    return pair.value;
                }
            }
        }
    }
}

/**
 * @author Jafar Amini in October 2018.
 */
class PairValueTitle {
    constructor(value, title) {
        this.value = value;
        this.title = title;
    }
    static getTitle(value, pairValueTitleList) {
        if (pairValueTitleList) {
            for (let pair of pairValueTitleList) {
                if (pair.value === value) {
                    return pair.title;
                }
            }
        }
    }
}

/**
 * @author Jafar Amini in February 2019.
 */
class PairNameValue {
    constructor(name, value) {
        this.name = name;
        this.value = value;
    }
    static getValue(name, pairNameValueList) {
        if (pairNameValueList) {
            for (let pair of pairNameValueList) {
                if (pair.name === name) {
                    return pair.value;
                }
            }
        }
    }
}

/**
 * @author Jafar Amini in October 2018.
 */
class PairIdTitle {
    constructor(id, title) {
        this.id = id;
        this.title = title;
    }
    static getTitle(id, pairIdTitleList) {
        if (pairIdTitleList) {
            for (let pair of pairIdTitleList) {
                if (pair.id === id) {
                    return pair.title;
                }
            }
        }
    }
}

/**
 * @author Jafar Amini in October 2018.
 */
class PairKeyTitle {
    constructor(key, title) {
        this.key = key;
        this.title = title;
    }
    static getTitle(key, pairKeyTitleList) {
        if (pairKeyTitleList) {
            for (let pair of pairKeyTitleList) {
                if (pair.key === key) {
                    return pair.title;
                }
            }
        }
    }
}

/**
 * Created by Jafar Amini in May 2018.
 */
class PairKeyValueT {
    constructor(key, value) {
        this.key = key;
        this.value = value;
    }
    getValue(key, pairKeyValueList) {
        if (pairKeyValueList) {
            for (const pair of pairKeyValueList) {
                if (pair.key === key) {
                    return pair.value;
                }
            }
        }
    }
}

class ObjectUtilT {
    getObject2KeyValArray(value) {
        const keys = [];
        for (const key in value) {
            keys.push(new PairKeyValueT(key, value[key]));
        }
        return keys;
    }
}

/**
 * Created by Jafar Amini in May 2018.
 */
class PairValueTitleT {
    constructor(value, title) {
        this.value = value;
        this.title = title;
    }
}

class CategorizedPairKeyValue {
    constructor(init = {}) {
        this.category = init.category || null;
        this.pairKeyValueList = init.pairKeyValueList || [];
    }
}

class CategorizedPairValueTitle {
    constructor(init = {}) {
        this.category = init.category || null;
        this.pairValueTitleList = init.pairValueTitleList || [];
    }
}

class CategorizedPairNameValue {
    constructor(init = {}) {
        this.category = init.category || null;
        this.pairNameValueList = init.pairNameValueList || [];
    }
}

/**
 * @author Jafar Amini in March 2018.
 */
class StringUtil {
    static getShorten(text, length, extra) {
        let exText = '';
        // alert("exText: " + exText);
        if (!text || (text && text.length === 0)) {
            return '';
        }
        else {
            if (text.length > length) {
                exText = !isNullOrUndefined(extra) ? ' ' + extra : '...';
                return text.substr(0, length) + exText;
            }
            else {
                return text.substr(0, length);
            }
        }
        // return text.substr(0,length,)+' '+ !isNullOrUndefined(extra) ? extra : "";
    }
    static getMatchesByRegix(str, re) {
        const matches = new Array();
        let m;
        matches[0] = [];
        matches[1] = [];
        do {
            m = re.exec(str);
            if (m) {
                matches[0].push(m[0]);
                matches[1].push(m[1]);
            }
        } while (m);
        // console.log(matches);
        return matches;
    }
    // لیستی از آبجکت‌های ساده (فاقد تودرتویی) را می‌گیرد و رشته کوئری مربوطه را برمی‌گرداند.
    static objectListToQueryString(objectList) {
        const finalQueryString = [];
        for (const object of objectList) {
            finalQueryString.push(this.objectToQueryString(object));
        }
        // return finalQueryString.join('&');
        return finalQueryString.filter(item => (item !== '')).join('&');
    }
    // یک آبجکت ساده (فاقد تودرتویی) را می‌گیرد و رشته-کوئری مربوطه را برمی‌گرداند.
    static objectToQueryString(object) {
        if (isNullOrUndefined(object))
            return '';
        // let props: Pair1[];
        let props = new Array();
        Object.keys(object).forEach(key => {
            props.push(new PairKeyValue(key, object[key]));
            // console.log(key);
        });
        const result = [];
        for (let i = 0; i < props.length; i++) {
            // console.log(i + ' ---> ', props[i]);
            result.push([props[i].key, props[i].value]);
        }
        //
        // if (params.offset != null) {
        //   result.push(['_start', params.offset]);
        // }
        // if (params.limit != null) {
        //   result.push(['_limit', params.limit]);
        // }
        // if (params.sortBy != null) {
        //   result.push(['_sort', params.sortBy]);
        // }
        // if (params.sortAsc != null) {
        //   result.push(['_order', params.sortAsc ? 'ASC' : 'DESC']);
        // }
        //
        return result.map(param => param.join('=')).join('&');
    }
    // یک آبجکت ساده (فاقد تودرتویی) و یا یک آبجکت مرکب
    // (دارای تودرتویی) را می‌گیرد و رشته-کوئری مربوطه را برمی‌گرداند.
    static objectToQueryString2(object) {
        let finalResult = [];
        this.object2QueryString(finalResult, object);
        return finalResult.filter(item => (item !== '')).join('&');
    }
    static object2QueryString(finalResult, object) {
        if (isNullOrUndefined(object))
            return '';
        // let props: Pair1[];
        let props = new Array();
        Object.keys(object).forEach(key => {
            // alert('key:.... ' + key + '   object[key] = ' + object[key]);
            if (!isNullOrUndefined(object[key])) {
                if (object[key].toString() === '[object Object]') {
                    // alert('Wawwwww');
                    this.object2QueryString(finalResult, object[key]);
                }
                else {
                    props.push(new PairKeyValue(key, object[key]));
                }
            }
            // console.log(key);
        });
        const result = [];
        for (let i = 0; i < props.length; i++) {
            // console.log(i + ' ---> ', props[i]);
            result.push([props[i].key, props[i].value]);
        }
        let r = result.map(param => param.join('=')).join('&');
        finalResult.push(r);
        return r;
    }
    static replaceRange(str, start, end, substitute) {
        return str.substring(0, start) + substitute + str.substring(end);
    }
    findClosingBracketMatchIndex(str, options) {
        if (str.substr(options.pos, options.fExp.length) !== options.fExp) {
            throw new Error('No ' + options.fExp + ' at index ' + options.pos);
        }
        let depth = 1;
        for (let i = options.pos + 1; i < str.length; i++) {
            if (str.substr(i, options.fExp.length) === options.fExp) {
                depth++;
            }
            if (str.substr(i, options.lExp.length) === options.lExp) {
                if (--depth === 0) {
                    return i;
                }
            }
        }
        return -1; // No matching closing parenthesis
    }
    static putAtIndex(str, item, index) {
        return str.slice(0, index) + item + str.slice(index);
    }
    /**
     *
     * @param str: string to  split
     * @param separator: split based on this
     * @param index: index of splitted array to return value
     * @param beginDirection: 'fromStart' | 'fromEnd'
     */
    static splitAndReturnItemByIndex(str, separator, index, beginDirection = 'fromStart') {
        const arr = str.split(separator);
        let res = null;
        if (arr.length > 0) {
            if (index > arr.length - 1) {
                throw new Error('index is out of bound of splitted array; array length: ' + arr.length + ', index: ' + index);
            }
            else {
                if (beginDirection === 'fromStart') {
                    res = arr[index];
                }
                else if (beginDirection === 'fromEnd') {
                    res = arr[arr.length - 1 - index];
                }
            }
        }
        else {
            res = null;
        }
        return res;
    }
}

let ShortenPipe = class ShortenPipe {
    transform(value, length, extra) {
        return value ? StringUtil.getShorten(value, length, extra) : '';
    }
};
ShortenPipe = __decorate([
    Pipe({
        name: 'shorten'
    })
], ShortenPipe);

let PipeStringModule = class PipeStringModule {
};
PipeStringModule = __decorate([
    NgModule({
        imports: [
            CommonModule
        ],
        declarations: [ShortenPipe],
        exports: [
            ShortenPipe
        ]
    })
], PipeStringModule);

/**
 * @author Jafar Amini in March 2018.
 */
var NotifyType;
(function (NotifyType) {
    NotifyType[NotifyType["SUCCESS"] = 'success'] = "SUCCESS";
    NotifyType[NotifyType["INFO"] = 'info'] = "INFO";
    NotifyType[NotifyType["WARNING"] = 'warning'] = "WARNING";
    NotifyType[NotifyType["DANGER"] = 'danger'] = "DANGER";
})(NotifyType || (NotifyType = {}));

/**
 * @author Jafar Amini in March 2018.
 */
class NotifyPlacement {
    constructor(from, align) {
        this.from = from;
        this.align = align;
    }
}
NotifyPlacement.Placement = {
    TOP_LEFT: new NotifyPlacement('top', 'left'),
    TOP_CENTER: new NotifyPlacement('top', 'center'),
    TOP_RIGHT: new NotifyPlacement('top', 'right'),
    BOTTOM_LEFT: new NotifyPlacement('bottom', 'left'),
    BOTTOM_CENTER: new NotifyPlacement('bottom', 'center'),
    BOTTOM_RIGHT: new NotifyPlacement('bottom', 'right')
};

/**
 * @author Jafar Amini in March 2018.
 */
var TEMPLATES;
(function (TEMPLATES) {
    TEMPLATES[TEMPLATES["Template1"] = 'template1'] = "Template1";
    TEMPLATES[TEMPLATES["Template2"] = 'template2'] = "Template2";
})(TEMPLATES || (TEMPLATES = {}));

/**
 * @author Jafar Amini in March 2018.
 */
class Notify {
    static showNotify(notifyConfig, selector) {
        // $.notify(notifyConfig.options, notifyConfig.settings);
        if (isNullOrUndefined(selector)) {
            $.notify(notifyConfig.options, notifyConfig.settings);
        }
        else {
            $(selector).notify(notifyConfig.options, notifyConfig.settings);
        }
    }
}
// public static Direction = {
//   TOP_LEFT: new NotifyDirection('top', 'left'),
//   TOP_CENTER: new NotifyDirection('top', 'center'),
//   TOP_RIGHT: new NotifyDirection('top', 'right'),
//   BOTTOM_LEFT: new NotifyDirection('bottom', 'left'),
//   BOTTOM_CENTER: new NotifyDirection('bottom', 'center'),
//   BOTTOM_RIGHT: new NotifyDirection('bottom', 'right')
// }
Notify.Placement = NotifyPlacement.Placement;
Notify.Type = NotifyType;
Notify.TEMPLATES = TEMPLATES;

/**
 * @author Jafar Amini in March 2018.
 */
var template1 = {
    'template1': '<div data-notify="container" class="col-xs-11 col-sm-3 alert alert-{0}" role="alert">' +
        // '<button type="button" aria-hidden="true" class="close pull-left" data-notify="dismiss">×</button>' +
        '<i data-notify="icon"></i> ' +
        '<span data-notify="title"style="font-weight: bold;">{1}</span> ' +
        '<br/>' +
        '<span data-notify="message">{2}</span>' +
        '<div class="progress" data-notify="progressbar">' +
        '<div class="progress-bar progress-bar-{0}" role="progressbar" aria-valuenow="0"' +
        ' aria-valuemin="0" aria-valuemax="100" style="width: 0%;"></div>' +
        '</div>' +
        '<a href="{3}" target="{4}" data-notify="url"></a>' +
        '</div>',
    'template1_without_title': '<div data-notify="container" class="col-xs-11 col-sm-3 alert alert-{0}" role="alert">' +
        // '<button type="button" aria-hidden="true" class="close pull-left" data-notify="dismiss">×</button>' +
        '<i data-notify="icon"></i> ' +
        '<span data-notify="message">{2}</span>' +
        '<div class="progress" data-notify="progressbar">' +
        '<div class="progress-bar progress-bar-{0}" role="progressbar" aria-valuenow="0"' +
        ' aria-valuemin="0" aria-valuemax="100" style="width: 0%;"></div>' +
        '</div>' +
        '<a href="{3}" target="{4}" data-notify="url"></a>' +
        '</div>',
    'template2': '<div data-notify="container" class="col-xs-11 col-sm-3 alert alert-{0}" role="alert">' +
        '<i data-notify="icon"  style="position: absolute; right: -30px; font-size: x-large;"></i>' +
        '<span data-notify="title"style="font-weight: bold;">{1}</span>' +
        '<br/>' +
        '<span data-notify="message"><div style="text-align: justify; text-justify: inter-word;">{2}</div></span>' +
        '<div class="progress" data-notify="progressbar">' +
        '<div class="progress-bar progress-bar-{0}" role="progressbar" aria-valuenow="0" ' +
        'aria-valuemin="0" aria-valuemax="100" style="width: 0%;"></div>' +
        '</div>' +
        '<a href="{3}" target="{4}" data-notify="url"></a>' +
        '</div>',
    'template2_without_title': '<div data-notify="container" class="col-xs-11 col-sm-3 alert alert-{0}" role="alert">' +
        '<div>' +
        '<i data-notify="icon" style="position: absolute; right: -30px; font-size: x-large;"></i>' +
        '</div>' +
        '<div>' +
        '<span data-notify="message">{2}</span>' +
        '</div>' +
        '<div class="progress" data-notify="progressbar">' +
        '<div class="progress-bar progress-bar-{0}" role="progressbar" aria-valuenow="0" ' +
        'aria-valuemin="0" aria-valuemax="100" style="width: 0%;"></div>' +
        '</div>' +
        '<a href="{3}" target="{4}" data-notify="url"></a>' +
        '</div>'
};

/**
 * @author Jafar Amini in March 2018.
 */
class Animate {
}
Animate.Attention_Seekers = {
    Bounce: 'animated bounce',
    Flash: 'animated flash',
    Pulse: 'animated pulse',
    RubberBand: 'animated rubberBand',
    Shake: 'animated shake',
    Swing: 'animated swing',
    Tada: 'animated tada',
    Wobble: 'animated wobble',
    Jello: 'animated jello'
};
Animate.Bouncing_Entrances = {
    BounceIn: 'animated bounceIn',
    BounceInDown: 'animated bounceInDown',
    BounceInLeft: 'animated bounceInLeft',
    BounceInRight: 'animated bounceInRight',
    BounceInUp: 'animated bounceInUp',
};
Animate.Bouncing_Exits = {
    BounceOut: 'animated bounceOut',
    BounceOutDown: 'animated bounceOutDown',
    BounceOutLeft: 'animated bounceOutLeft',
    BounceOutRight: 'animated bounceOutRight',
    BounceOutUp: 'animated bounceOutUp'
};
Animate.Fading_Entrances = {
    FadeIn: 'animated fadeIn',
    FadeInDown: 'animated fadeInDown',
    FadeInDownBig: 'animated fadeInDownBig',
    FadeInLeft: 'animated fadeInLeft',
    FadeInLeftBig: 'animated fadeInLeftBig',
    FadeInRight: 'animated fadeInRight',
    FadeInRightBig: 'animated fadeInRightBig',
    FadeInUp: 'animated fadeInUp',
    FadeInUpBig: 'animated fadeInUpBig'
};
Animate.Fading_Exits = {
    FadeOut: 'animated fadeOut',
    FadeOutDown: 'animated fadeOutDown',
    FadeOutDownBig: 'animated fadeOutDownBig',
    FadeOutLeft: 'animated fadeOutLeft',
    FadeOutLeftBig: 'animated fadeOutLeftBig',
    FadeOutRight: 'animated fadeOutRight',
    FadeOutRightBig: 'animated fadeOutRightBig',
    FadeOutUp: 'animated fadeOutUp',
    FadeOutUpBig: 'animated fadeOutUpBig'
};
Animate.Flippers = {
    Flip: 'animated flip',
    FlipInX: 'animated flipInX',
    FlipInY: 'animated flipInY',
    FlipOutX: 'animated flipOutX',
    FlipOutY: 'animated flipOutY',
};
Animate.Lightspeed = {
    LightSpeedIn: 'animated lightSpeedIn',
    LightSpeedOut: 'animated lightSpeedOut'
};
Animate.Rotating_Entrances = {
    RotateIn: 'animated rotateIn',
    RotateInDownLeft: 'animated rotateInDownLeft',
    RotateInDownRight: 'animated rotateInDownRight',
    RotateInUpLeft: 'animated rotateInUpLeft',
    RotateInUpRight: 'animated rotateInUpRight'
};
Animate.Rotating_Exits = {
    RotateOut: 'animated rotateOut',
    RotateOutDownLeft: 'animated rotateOutDownLeft',
    RotateOutDownRight: 'animated rotateOutDownRight',
    RotateOutUpLeft: 'animated rotateOutUpLeft',
    RotateOutUpRight: 'animated rotateOutUpRight'
};
Animate.Sliding_Entrances = {
    SlideInUp: 'animated slideInUp',
    SlideInDown: 'animated slideInDown',
    SlideInLeft: 'animated slideInLeft',
    SlideInRight: 'animated slideInRight',
};
Animate.Sliding_Exits = {
    SlideOutUp: 'animated slideOutUp',
    SlideOutDown: 'animated slideOutDown',
    SlideOutLeft: 'animated slideOutLeft',
    SlideOutRight: 'animated slideOutRight',
};
Animate.Zoom_Entrances = {
    ZoomIn: 'animated zoomIn',
    ZoomInDown: 'animated zoomInDown',
    ZoomInLeft: 'animated zoomInLeft',
    ZoomInRight: 'animated zoomInRight',
    ZoomInUp: 'animated zoomInUp'
};
Animate.Zoom_Exits = {
    ZoomOut: 'animated zoomOut',
    ZoomOutDown: 'animated zoomOutDown',
    ZoomOutLeft: 'animated zoomOutLeft',
    ZoomOutRight: 'animated zoomOutRight',
    ZoomOutUp: 'animated zoomOutUp',
};
Animate.Specials = {
    Hinge: 'animated hinge',
    JackInTheBox: 'animated jackInTheBox',
    RollIn: 'animated rollIn',
    RollOut: 'animated rollOut',
};

/**
 * @author Jafar Amini in March 2018.
 */
var ConfigDefault = {
    options: {
        icon: null,
        title: null,
        message: null,
        url: null,
        target: null // '_blank'
    },
    settings: {
        element: 'body',
        position: null,
        type: NotifyType.INFO,
        allow_dismiss: false,
        newest_on_top: false,
        showProgressbar: false,
        placement: {
            from: 'top',
            align: 'left'
        },
        offset: 35,
        spacing: 10,
        z_index: 10000,
        delay: 4000,
        timer: 1000,
        url_target: null,
        mouse_over: null,
        animate: {
            enter: Animate.Fading_Entrances.FadeInLeft,
            exit: Animate.Fading_Exits.FadeOutLeft
        },
        onShow: null,
        onShown: null,
        onClose: null,
        onClosed: null,
        icon_type: 'class',
        template: template1['template1']
    }
};

/**
 * @author Jafar Amini in March 2018.
 */
class NotifyConfig {
    constructor(notifyType, notifyPlacement, template, message, title) {
        this.settings = ConfigDefault.settings;
        this.options = ConfigDefault.options;
        this.options.title = title;
        this.options.message = message;
        if (!isNullOrUndefined(template) && title !== '') {
            if (isNullOrUndefined(title) || title === '') {
                if (!this.settings.hasOwnProperty('template')) {
                    this.settings['template'] = '';
                }
                this.settings.template = template1[template + '_without_title'];
            }
            else {
                this.settings.template = template1[template];
            }
        }
        else {
            delete this.settings.template;
        }
        this.settings.type = notifyType;
        this.settings.placement.from = notifyPlacement.from;
        this.settings.placement.align = notifyPlacement.align;
        switch (notifyType) {
            case NotifyType.INFO:
                this.options.icon = 'fa fa-info-circle';
                this.settings.animate.enter = Animate.Fading_Entrances.FadeInLeft;
                this.settings.animate.exit = Animate.Fading_Exits.FadeOutLeft;
                break;
            case NotifyType.WARNING:
                this.options.icon = 'fa fa-exclamation-triangle';
                this.settings.animate.enter = Animate.Bouncing_Entrances.BounceIn;
                this.settings.animate.exit = Animate.Bouncing_Exits.BounceOut;
                break;
            case NotifyType.SUCCESS:
                this.options.icon = 'fa fa-check-circle';
                this.settings.animate.enter = Animate.Fading_Entrances.FadeInLeft;
                this.settings.animate.exit = Animate.Fading_Exits.FadeOutLeft;
                break;
            case NotifyType.DANGER:
                this.options.icon = 'fa fa-exclamation-triangle';
                this.settings.animate.enter = Animate.Bouncing_Entrances.BounceIn;
                this.settings.animate.exit = Animate.Bouncing_Exits.BounceOut;
                break;
        }
    }
}
class NotifyConfigSettings {
}
class NotifyConfigOptions {
}

/**
 * @author Jafar Amini in March 2018.
 */
var ActionMode;
(function (ActionMode) {
    ActionMode[ActionMode["ADD"] = 'ADD'] = "ADD";
    ActionMode[ActionMode["EDIT"] = 'EDIT'] = "EDIT";
    ActionMode[ActionMode["VIEW"] = 'VIEW'] = "VIEW";
})(ActionMode || (ActionMode = {}));

var ElementType;
(function (ElementType) {
    ElementType[ElementType["INPUT"] = 'INPUT'] = "INPUT";
    ElementType[ElementType["SELECT"] = 'SELECT'] = "SELECT";
    ElementType[ElementType["SPAN"] = 'SPAN'] = "SPAN";
})(ElementType || (ElementType = {}));

var OrderDirection;
(function (OrderDirection) {
    OrderDirection[OrderDirection["asc"] = 'asc'] = "asc";
    OrderDirection[OrderDirection["desc"] = 'desc'] = "desc";
})(OrderDirection || (OrderDirection = {}));

var QueryOperatorEnum;
(function (QueryOperatorEnum) {
    QueryOperatorEnum[QueryOperatorEnum["equals"] = 0] = "equals";
    QueryOperatorEnum[QueryOperatorEnum["notEquals"] = 1] = "notEquals";
    QueryOperatorEnum[QueryOperatorEnum["equalsIgnoreCase"] = 2] = "equalsIgnoreCase";
    QueryOperatorEnum[QueryOperatorEnum["notEqualsIgnoreCase"] = 3] = "notEqualsIgnoreCase";
    QueryOperatorEnum[QueryOperatorEnum["lessThan"] = 4] = "lessThan";
    QueryOperatorEnum[QueryOperatorEnum["greaterThan"] = 5] = "greaterThan";
    QueryOperatorEnum[QueryOperatorEnum["lessThanOrEquals"] = 6] = "lessThanOrEquals";
    QueryOperatorEnum[QueryOperatorEnum["greaterThanOrEquals"] = 7] = "greaterThanOrEquals";
    QueryOperatorEnum[QueryOperatorEnum["like"] = 8] = "like";
})(QueryOperatorEnum || (QueryOperatorEnum = {}));

/**
 * @author Jafar Amini in March 2018.
 */
var RequestStatus;
(function (RequestStatus) {
    RequestStatus[RequestStatus["NONE"] = 'NONE'] = "NONE";
    RequestStatus[RequestStatus["PENDING"] = 'PENDING'] = "PENDING";
    RequestStatus[RequestStatus["SUCCESS"] = 'SUCCESS'] = "SUCCESS";
    RequestStatus[RequestStatus["ERROR"] = 'ERROR'] = "ERROR";
})(RequestStatus || (RequestStatus = {}));

/**
 * @author Jafar Amini in March 2018.
 */
class ActionModeUtil {
    static isEditOrAddMode(myMode_I) {
        return !isNullOrUndefined(myMode_I)
            && (myMode_I === ActionMode.EDIT || myMode_I === ActionMode.ADD);
    }
    static isEditOrViewMode(myMode_I) {
        return !isNullOrUndefined(myMode_I)
            && (myMode_I === ActionMode.EDIT || myMode_I === ActionMode.VIEW);
    }
    static isAddOrViewMode(myMode_I) {
        return !isNullOrUndefined(myMode_I)
            && (myMode_I === ActionMode.ADD || myMode_I === ActionMode.VIEW);
    }
    static isAddMode(myMode_I) {
        return !isNullOrUndefined(myMode_I) && (myMode_I === ActionMode.ADD);
    }
    static isEditMode(myMode_I) {
        return !isNullOrUndefined(myMode_I)
            && (myMode_I === ActionMode.EDIT);
    }
    static isViewMode(myMode_I) {
        return !isNullOrUndefined(myMode_I)
            && (myMode_I === ActionMode.VIEW);
    }
}

/**
 * @author Jafar Amini in March 2018.
 */
class BooleanMap {
    static yesNo(value) {
        if (value === true) {
            return 'بله';
        }
        else if (value === false) {
            return 'خیر';
        }
    }
    static activeDeactive(value) {
        if (value === true) {
            return 'فعال';
        }
        else if (value === false) {
            return 'غیرفعال';
        }
    }
    static has_Not(value) {
        if (value === true) {
            return 'دارد';
        }
        else if (value === false) {
            return 'ندارد';
        }
    }
    static approved_DisApproved(value) {
        if (value === true) {
            return 'تأیید شد';
        }
        else if (value === false) {
            return 'رد شد';
        }
    }
}

class BootstrapUtil {
    static goToCrouselSlide(carouselSelector, slideNumber) {
        $(carouselSelector).carousel(slideNumber);
    }
}

/**
 * @author Jafar Amini in March 2018.
 */
class DatePattern {
}
DatePattern.pattern1_Jalali = 'jYYYY/jMM/jDD - HH:mm';
DatePattern.pattern1_Gregorian = 'YYYY/MM/DD - HH:mm';

/**
 * @author Jafar Amini in March 2018.
 */
class MathUtil {
    static toRadians(degree) {
        return degree * (Math.PI / 180);
    }
    static toDegrees(radian) {
        return radian * (180 / Math.PI);
    }
}

/**
 * Created by naser on 10/7/17.
 */
class Pattern {
}
Pattern.email = '^[_A-Za-z0-9-\\+]+(\\.[_A-Za-z0-9-]+)*@[A-Za-z0-9-]+(\\.[A-Za-z0-9]+)*(\\.[A-Za-z]{2,})$';
Pattern.phone = '(([0]+[9])([0-9]{9})||([۰]+[۹])([۰-۹]{9}))$';
Pattern.tell = '(([0])([0-9]{10})||([۰])([۰-۹]{10}))$';
Pattern.nationalCode = '(([0-9]{10})||([۰-۹]{10}))$';
Pattern.faText = '[آ-ی ]+';
Pattern.date = '^(([0-9]{1})([.,\/]([0-9]{1,2})){0,1}|([۰-۹]{1})([.,\/]([۰-۹]{1,2})){0,1})';
Pattern.streetName = '[۰-۹-0-9آ-ی ]+';
Pattern.bankCard = '((([1-9])([0-9]{18})||([۱-۹])([۰-۹]{18}))||(([1-9])([0-9]{15})||([۱-۹])([۰-۹]{15})))$';
Pattern.bcNumber = '[1-9]{1}|[1-9][0-9]{2}|[۱-۹]{1}|[۱-۹][۰-۹]{2}]'; // شماره شناسنامه
Pattern.password = '[-_/@!#$%^&*().A-Za-z0-9]+';
Pattern.number = '[0-9]|[۰-۹]';
Pattern.faNumberAndText = '[۰-۹]*|[آ-ی ]*';
Pattern.postalCode = '((([1-9]{1})([0-9]{9}))||(([۱-۹]{1})([۰-۹]{9})))$';

class Select2Util {
    static changeSelectedList(referenceList, selectedList, value) {
        selectedList = [];
        if (value !== null) {
            for (const item of value) {
                for (const refItem of referenceList) {
                    if (refItem.id === item) {
                        selectedList.push(refItem.id);
                    }
                }
            }
        }
        return selectedList;
    }
    static resetSelectedList(selector) {
        $(selector).val([]);
        $(selector).trigger('change');
    }
}

/**
 * Edited by Jafar Amini in March 2018.
 */
// declare var swal: any;
class SwalUtil {
    static successAdd(title, text) {
        swal({
            title: !isNullOrUndefined(title)
                ? title :
                'با موفقیت ثبت شد.',
            text: !isNullOrUndefined(text)
                ? text :
                '',
            type: 'success',
            confirmButtonColor: '#04383c',
        });
    }
    static warningAdd(title, text) {
        swal({
            title: !isNullOrUndefined(title)
                ? title :
                'عملیات انجام نشد!',
            text: !isNullOrUndefined(text)
                ? text :
                'دوباره تلاش کنید.',
            type: 'error',
            confirmButtonColor: '#1EE8A8',
        });
    }
    static successEdit(title, text) {
        swal({
            title: !isNullOrUndefined(title)
                ? title :
                'با موفقیت ویرایش شد.',
            text: !isNullOrUndefined(text)
                ? text :
                '',
            type: 'success',
            confirmButtonColor: '#04383c',
        });
    }
    static warningEdit(title, text) {
        swal({
            title: !isNullOrUndefined(title)
                ? title :
                'عملیات انجام نشد!',
            text: !isNullOrUndefined(text)
                ? text :
                'دوباره تلاش کنید.',
            type: 'error',
            confirmButtonColor: '#1EE8A8',
        });
    }
}

/**
 * @author Jafar Amini in March 2018.
 */
class Validation {
    // ولیدیشن کد ملی
    static checkCodeMeli(value) {
        const code = Common.Fa2En(value);
        if (value.length >= 8) {
            const c = parseInt(code.substr(9, 1), 10);
            let s = 0;
            for (let i = 0; i < 9; i++) {
                s += parseInt(code.substr(i, 1), 10) * (10 - i);
            }
            s = s % 11;
            if ((s < 2 && c === s) || (s >= 2 && c === (11 - s))) {
                return true;
            }
        }
        return false;
    }
}

class CaseFormat {
    static camelCaseToLowerHyphen(myStr) {
        if (isNullOrUndefined(myStr)) {
            return myStr;
        }
        else {
            return myStr.replace(/([a-z])([A-Z])/g, '$1-$2').toLowerCase();
        }
    }
}

class ObjectUtil {
    static getObjectPropertyFromParamMap(object, route) {
        for (let key of route.paramMap.keys) {
            if (object.hasOwnProperty(key))
                object[key] = route.paramMap.get(key);
        }
    }
    static getObjectPropertyList(object) {
        let propertyList;
        propertyList = Object.getOwnPropertyNames(object);
        return propertyList;
    }
    static unFlatify(flatParams, dest, options) {
        return ObjectUtil.unFlatify2(flatParams, dest, null, options);
    }
    static unFlatify2(flatParams, dest, sample, options) {
        let convertCamelToLowerHyphen = false;
        if (isNullOrUndefined(options)) {
            convertCamelToLowerHyphen = false;
        }
        if (!isNullOrUndefined(options) && isNullOrUndefined(options.convertCamelToLowerHyphen)) {
            convertCamelToLowerHyphen = false;
        }
        if (!isNullOrUndefined(options) && !isNullOrUndefined(options.convertCamelToLowerHyphen)) {
            if (options.convertCamelToLowerHyphen) {
                convertCamelToLowerHyphen = true;
            }
            else {
                convertCamelToLowerHyphen = false;
            }
        }
        // CmnToolkit2.CaseFormat.camelCaseToLowerHyphen(key)
        Object.keys(dest).forEach(key => {
            // alert('key:.... ' + key + '   object[key] = ' + object[key]);
            if (!isNullOrUndefined(dest[key]) && dest[key].toString() === '[object Object]') {
                if (!isNullOrUndefined(sample)) {
                    this.unFlatify2(flatParams, dest[key], sample[key], { convertCamelToLowerHyphen: convertCamelToLowerHyphen });
                }
                else {
                    this.unFlatify2(flatParams, dest[key], null, { convertCamelToLowerHyphen: convertCamelToLowerHyphen });
                }
            }
            else {
                if (convertCamelToLowerHyphen) {
                    // alert(typeof sample[key]);
                    if (flatParams.hasOwnProperty(CaseFormat.camelCaseToLowerHyphen(key))) {
                        if (!isNullOrUndefined(sample) && !isNullOrUndefined(sample[key])) {
                            dest[key] = ObjectUtil.getConvertedType(typeof sample[key], flatParams[CaseFormat.camelCaseToLowerHyphen(key)]);
                        }
                        else {
                            dest[key] = flatParams[CaseFormat.camelCaseToLowerHyphen(key)];
                        }
                    }
                }
                else if (!convertCamelToLowerHyphen) {
                    if (flatParams.hasOwnProperty(key)) {
                        if (!isNullOrUndefined(sample) && !isNullOrUndefined(sample[key])) {
                            dest[key] = ObjectUtil.getConvertedType(typeof sample[key], flatParams[key]);
                        }
                        else {
                            dest[key] = flatParams[key];
                        }
                    }
                }
            }
        });
        return dest;
    }
    static getConvertedType(type, data) {
        switch (type) {
            case 'boolean':
                if (data === 'true') {
                    data = true;
                }
                else if (data === 'false') {
                    data = false;
                }
                else {
                    data = null;
                }
                break;
            case 'number':
                data = +data;
                break;
            case 'string':
                data = data.toString();
                break;
        }
        return data;
    }
    static deepClone(objectpassed) {
        if (objectpassed === null || typeof objectpassed !== 'object') {
            return objectpassed;
        }
        // give temporary-storage the original obj's constructor
        let temporary_storage = objectpassed.constructor();
        for (var key in objectpassed) {
            temporary_storage[key] = this.deepClone(objectpassed[key]);
        }
        return temporary_storage;
    }
    static circularDeepClone(obj) {
        var visitedNodes = [];
        var clonedCopy = [];
        function clone(item) {
            if (typeof item === 'object' && !Array.isArray(item)) {
                if (visitedNodes.indexOf(item) === -1) {
                    visitedNodes.push(item);
                    var cloneObject = {};
                    clonedCopy.push(cloneObject);
                    for (var i in item) {
                        if (item.hasOwnProperty(i)) {
                            cloneObject[i] = clone(item[i]);
                        }
                    }
                    return cloneObject;
                }
                else {
                    return clonedCopy[visitedNodes.indexOf(item)];
                }
            }
            else if (typeof item === 'object' && Array.isArray(item)) {
                if (visitedNodes.indexOf(item) === -1) {
                    var cloneArray = [];
                    visitedNodes.push(item);
                    clonedCopy.push(cloneArray);
                    for (var j = 0; j < item.length; j++) {
                        cloneArray.push(clone(item[j]));
                    }
                    return cloneArray;
                }
                else {
                    return clonedCopy[visitedNodes.indexOf(item)];
                }
            }
            return item; // not object, not array, therefore primitive
        }
        return clone(obj);
    }
    // // Test deepClone
    // let employeeDetailsOriginal =
    //   {
    //     name: 'Manjula', age: 25, Profession: 'Software Engineer', eye: {
    //       eyeNumber: 9,
    //       color: 'Blue'
    //     }
    //   };
    // let employeeDetailsDuplicate = (Utils.deepClone(employeeDetailsOriginal));
    // employeeDetailsDuplicate.name = 'NameChanged';
    // employeeDetailsOriginal.eye.color = 'Black';
    // console.log('employeeDetailsOriginal', employeeDetailsOriginal);
    // console.log('employeeDetailsDuplicate', employeeDetailsDuplicate);
    static overlayValues(source, dest) {
        Object.keys(source).forEach(key => {
            // alert('key:.... ' + key + '   object[key] = ' + object[key]);
            if (!isNullOrUndefined(source[key]) && source[key].toString() === '[object Object]') {
                this.overlayValues(source[key], dest[key]);
            }
            else {
                dest[key] = source[key];
            }
        });
        return dest;
    }
    static overlayValuesClearDestObjects(source, dest) {
        Object.keys(source).forEach(key => {
            // alert('key:.... ' + key + '   object[key] = ' + object[key]);
            if (!isNullOrUndefined(source[key]) && source[key].toString() === '[object Object]') {
                this.overlayValues(source[key], dest[key]);
            }
            else {
                dest[key] = source[key];
            }
        });
        return dest;
    }
    // یک آبجکت ساده یا مرکب (دارای تودرتویی) را می‌گیرد و یک آبجکت ساده بدون تودرتویی تحویل می‌دهد.
    static FlatiFy(newObject, object, removeIfEmpty) {
        Object.keys(object).forEach(key => {
            if (!isNullOrUndefined(object[key]) && object[key].toString() === '[object Object]') {
                this.FlatiFy(newObject, object[key], removeIfEmpty);
            }
            else {
                if (removeIfEmpty === true && object[key] === '') {
                    newObject[key] = null;
                }
                else {
                    newObject[key] = object[key];
                }
            }
        });
        return newObject;
    }
    static getObject2KeyValArray(value) {
        let keys = [];
        for (let key in value) {
            keys.push(new PairKeyValue(key, value[key]));
        }
        return keys;
    }
    /**
     *
     * @param obj
     * @param desc
     * Example:
     var obj = {a: {b: {c: 0}}};
     var propPath = getPropPath();  // returns e.g. "a.b.c"
     var result = getDescendantProp(obj, propPath);
     console.log(result); // output: 0
     */
    static getDescendantProp(obj, propPath) {
        var arr = propPath.split('.');
        while (arr.length) {
            obj = obj[arr.shift()];
        }
        return obj;
    }
    /**
     *
     * @param obj
     * @param desc
     * @param value
     *
     Example:
     var obj = {a: {b: {c: 0}}};
     var propPath = getPropPath();  // returns e.g. "a.b.c"
     var result = setDescendantProp(obj, propPath, 1);  // test.a.b.c will now be 1
     */
    static setDescendantProp(obj, propPath, value) {
        var arr = propPath.split('.');
        while (arr.length > 1) {
            obj = obj[arr.shift()];
        }
        return obj[arr[0]] = value;
    }
    /**
     *
     * @param obj
     * @param desc
     * @param value
     *
     Example:
     var obj = {a: {b: {c: 0}}};
     var propPath = getPropPath();  // returns e.g. "a.b.c"
     var result = pushDescendantProp(obj, propPath, 1);  // test.a.b.c will now be 1
     */
    static pushDescendantProp(obj, propPath, value) {
        var arr = propPath.split('.');
        while (arr.length > 1) {
            obj = obj[arr.shift()];
        }
        return obj[arr[0]].push(value);
    }
    /**
     *
     * @param str
     * @param val
     * @param obj
     * example: str= 'a.b.c', val = {"title":"alef", code: 2}, obj = {} --> result: obj= a: {b: {c: {"title":"alef", code: 2}}}
     */
    static parseDotNotationAndSetValue(str, val, obj) {
        let currentObj = obj;
        const keys = str.split('.');
        let i, key;
        const l = Math.max(1, keys.length - 1);
        for (i = 0; i < l; ++i) {
            key = keys[i];
            currentObj[key] = currentObj[key] || {};
            currentObj = currentObj[key];
        }
        currentObj[keys[i]] = val;
        delete obj[str];
    }
    static listToObject(list, getKey) {
        const obj = {};
        if (list && list.length && list.length > 0) {
            for (const item of list) {
                obj[getKey(item)] = item;
            }
        }
        return obj;
    }
}

class JsonUtil {
    static isJsonString(str) {
        try {
            JSON.parse(str);
        }
        catch (e) {
            return false;
        }
        return true;
    }
    static convertFlatObjectToPairArray(obj, keyString, valString) {
        let props = this.toPairArray(obj);
        let finalArray = [];
        for (let i = 0; i < props.length; i++) {
            finalArray.push({ name: props[i].key, value: props[i].value });
        }
        return finalArray;
        // for (let key in obj) {
        //   console.log('      key:', key, 'value:', obj[key]);
        // }
    }
    static toPairArray(object) {
        let props = new Array();
        Object
            .keys(object)
            .forEach(key => {
            if (!isNullOrUndefined(object[key])) {
                if (object[key].toString() === '[object Object]') {
                    this.toPairArray(object[key]);
                }
                else {
                    props.push(new PairKeyValue(key, object[key]));
                }
            }
            // console.log(key);
        });
        return props;
        // for (let key in obj) {
        //   console.log('      key:', key, 'value:', obj[key]);
        // }
    }
}

var ReadMode;
(function (ReadMode) {
    ReadMode[ReadMode["Text"] = 0] = "Text";
    ReadMode[ReadMode["ArrayBuffer"] = 1] = "ArrayBuffer";
    ReadMode[ReadMode["BinaryString"] = 2] = "BinaryString";
    ReadMode[ReadMode["DataURL"] = 3] = "DataURL";
})(ReadMode || (ReadMode = {}));

class FileUtil {
    static prepareUploadOne(formData, event, multiparKey) {
        let myMultiparKey = '';
        if (isNullOrUndefined(multiparKey)) {
            myMultiparKey = 'file';
        }
        else {
            myMultiparKey = multiparKey;
        }
        const x = event.target.files[0];
        formData.append(myMultiparKey, x);
        formData.append('count', '1');
    }
    static prepareUploadMulti(formData, event, multiparKey) {
        let myMultiparKey = '';
        if (isNullOrUndefined(multiparKey)) {
            myMultiparKey = 'files';
        }
        else {
            myMultiparKey = multiparKey;
        }
        for (let i = 0; i < event.target.files.length; i++) {
            formData.append(myMultiparKey, event.target.files[i]);
        }
        formData.append('count', event.target.files.length);
    }
    static readFile(file, done, readMod) {
        if (!file) {
            return done(new Error('no file chosen'));
        }
        const reader = new FileReader();
        reader.onload = () => {
            done(null, reader.result);
        };
        switch (readMod) {
            case ReadMode.Text:
                reader.readAsText(file);
                break;
            case ReadMode.ArrayBuffer:
                reader.readAsArrayBuffer(file);
                break;
            case ReadMode.BinaryString:
                reader.readAsBinaryString(file);
                break;
            case ReadMode.DataURL:
                reader.readAsDataURL(file);
                break;
            default:
                reader.readAsText(file);
        }
    }
}

class Spesific {
    static fa2finglish(str) {
        return str.replace('آ', 'a')
            .replace('أ', 'a')
            .replace('إ', 'e')
            .replace('ب', 'b')
            .replace('پ', 'p')
            .replace('ت', 't')
            .replace('ث', 's')
            .replace('ج', 'j')
            .replace('چ', 'ch')
            .replace('ح', 'h')
            .replace('خ', 'kh')
            .replace('د', 'd')
            .replace('ذ', 'z')
            .replace('ر', 'r')
            .replace('ز', 'z')
            .replace('ژ', 'zh')
            .replace('س', 's')
            .replace('ش', 'sh')
            .replace('ص', 's')
            .replace('ض', 'z')
            .replace('ط', 't')
            .replace('ظ', 'z')
            .replace('ع', 'ea')
            .replace('غ', 'gh')
            .replace('ف', 'f')
            .replace('ق', 'gh')
            .replace('ک', 'k')
            .replace('گ', 'g')
            .replace('ل', 'l')
            .replace('م', 'm')
            .replace('ن', 'n')
            .replace('و', 'v')
            .replace('ه', 'h')
            .replace('ی', 'y')
            .replace('ي', 'y')
            .replace('ئ', 'wa')
            .replace('ء', 'wa')
            .replace('ة', 't')
            .replace('ؤ', 'w')
            .replace('؟', '?')
            .replace('ك', 'k')
            .replace('ّ', '')
            .replace('َ', '')
            .replace('ِ', '')
            .replace('ُ', '')
            .replace('ً', '')
            .replace('ٍ', '')
            .replace('ٌ', '')
            .replace('ٌ', '')
            .replace('ْ', '')
            .replace('ٓ', '')
            .replace('ٰ', '')
            .replace('ٔ', '')
            .replace('‌', '');
    }
}

class WindowMediaUtil {
    static isXl() {
        if (WindowMediaUtil.matchXl()) {
            return true;
        }
        return false;
    }
    static isLg() {
        if (!WindowMediaUtil.matchXl() && WindowMediaUtil.matchLg()) {
            return true;
        }
        return false;
    }
    static isMd() {
        if (!WindowMediaUtil.matchXl() && !WindowMediaUtil.matchLg() && WindowMediaUtil.matchMd()) {
            return true;
        }
        return false;
    }
    static isSm() {
        if (!WindowMediaUtil.matchXl() && !WindowMediaUtil.matchLg() && !WindowMediaUtil.matchMd() && WindowMediaUtil.matchSm()) {
            return true;
        }
        return false;
    }
    static isXs() {
        if (!WindowMediaUtil.matchXl() && !WindowMediaUtil.matchLg() && !WindowMediaUtil.matchMd() && !WindowMediaUtil.matchSm() && WindowMediaUtil.matchXs()) {
            return true;
        }
        return false;
    }
    static isSmOrSmaller() {
        if (WindowMediaUtil.isSm() || WindowMediaUtil.isXs()) {
            return true;
        }
        return false;
    }
    static isMdOrSmaller() {
        if (WindowMediaUtil.isMd() || WindowMediaUtil.isSm() || WindowMediaUtil.isXs()) {
            return true;
        }
        return false;
    }
    static isLgOrSmaller() {
        if (WindowMediaUtil.isLg() || WindowMediaUtil.isMd() || WindowMediaUtil.isSm() || WindowMediaUtil.isXs()) {
            return true;
        }
        return false;
    }
    static matchXl() {
        if (window.matchMedia('(min-width: 1200px)').matches) {
            return true;
        }
        return false;
    }
    static matchLg() {
        if (window.matchMedia('(min-width: 992px)').matches) {
            return true;
        }
        return false;
    }
    static matchMd() {
        if (window.matchMedia('(min-width: 768px)').matches) {
            return true;
        }
        return false;
    }
    static matchSm() {
        if (window.matchMedia('(min-width: 576px)').matches) {
            return true;
        }
        return false;
    }
    static matchXs() {
        if (window.matchMedia('(min-width: 0px)').matches) {
            return true;
        }
        return false;
    }
}

class ArrayUtil {
    static putAtIndex(arr, item, index) {
        arr.splice(index, 0, item).join();
    }
}

class DomUtil {
    static typeInElement(el, newText) {
        const start = el.prop('selectionStart');
        const end = el.prop('selectionEnd');
        const text = el.val();
        const before = text.substring(0, start);
        const after = text.substring(end, text.length);
        el.val(before + newText + after);
        el[0].selectionStart = el[0].selectionEnd = start + newText.length;
        el.focus();
    }
    static insertTextIntoElement(selector, resText, textToInsert, options) {
        $(selector).val(resText);
        if (options.addSpaceAfter === true) {
            DomUtil.typeInElement($(selector), textToInsert + ' ');
        }
        else {
            DomUtil.typeInElement($(selector), textToInsert);
        }
        resText = $(selector).val();
    }
}

/**
 * @author Jafar Amini in March 2018.
 */
var Toolkit2;
(function (Toolkit2) {
    Toolkit2.Notify = Notify;
    Toolkit2.NotifyConfig = NotifyConfig;
    Toolkit2.ActionModeUtil = ActionModeUtil;
    Toolkit2.Animate = Animate;
    Toolkit2.BooleanMap = BooleanMap;
    Toolkit2.BootstrapUtil = BootstrapUtil;
    Toolkit2.Common = Common;
    Toolkit2.DatePattern = DatePattern;
    Toolkit2.MathUtil = MathUtil;
    Toolkit2.Moment = Moment;
    Toolkit2.Pattern = Pattern;
    Toolkit2.Select2Util = Select2Util;
    Toolkit2.StringUtil = StringUtil;
    Toolkit2.SwalUtil = SwalUtil;
    Toolkit2.Validation = Validation;
    Toolkit2.ObjectUtil = ObjectUtil;
    Toolkit2.JsonUtil = JsonUtil;
    Toolkit2.CaseFormat = CaseFormat;
    Toolkit2.FileUtil = FileUtil;
    Toolkit2.Spesific = Spesific;
    Toolkit2.WindowMediaUtil = WindowMediaUtil;
    Toolkit2.ArrayUtil = ArrayUtil;
    Toolkit2.DomUtil = DomUtil;
})(Toolkit2 || (Toolkit2 = {}));

class UnFlatifyOptions {
}

/**
 * @author Jafar Amini in March 2018.
 */
class Interaction {
    static trackChange(changes) {
        let changeLog = [];
        console.log('-----------------------<changes>');
        console.log('changes: ', changes);
        let log = [];
        for (let propName in changes) {
            let changedProp = changes[propName];
            let to = JSON.stringify(changedProp.currentValue);
            if (changedProp.isFirstChange()) {
                log.push(`Initial value of ${propName} set to ${to}`);
            }
            else {
                let from = JSON.stringify(changedProp.previousValue);
                log.push(`${propName} changed from ${from} to ${to}`);
            }
        }
        changeLog.push(log.join(', '));
        console.log('***          |||<changeLog>|||          ***');
        console.log('changeLog ', changeLog);
        console.log('***          |||</changeLog>|||          ***');
        console.log('-----------------------</changes>');
    }
}

/**
 * @author Jafar Amini in March 2018.
 */
var CommonDevTools;
(function (CommonDevTools) {
    CommonDevTools.Interaction = Interaction;
})(CommonDevTools || (CommonDevTools = {}));

/**
 * @author Jafar Amini on 2018.
 */
class EnumHandle {
    /**
     *
     * @param myEnum
     */
    static getAsObject(myEnum) {
        let myOb = new Object();
        let prevKey, prevVal;
        for (let item in myEnum) {
            if (isNaN(Number(item))) {
                // console.log(item, myEnum[item]);
                if (item !== prevVal && myEnum[item] !== prevKey) {
                    myOb[item] = myEnum[item];
                }
                prevKey = item;
                prevVal = myEnum[item];
            }
        }
        return myOb;
    }
    static getAsValueTitleList(myEnum) {
        let value = EnumHandle.getAsObject(myEnum);
        let keys = [];
        for (let key in value) {
            // keys.push({value: key, title: value[key]});
            keys.push(new PairValueTitle(key, value[key]));
        }
        return keys;
    }
    static getAsValueList(myEnum) {
        let value = EnumHandle.getAsObject(myEnum);
        let keys = [];
        for (let key in value) {
            // keys.push({value: key, title: value[key]});
            keys.push(key);
        }
        return keys;
    }
    static getAsTitleList(myEnum) {
        let value = EnumHandle.getAsObject(myEnum);
        let keys = [];
        for (let key in value) {
            // keys.push({value: key, title: value[key]});
            keys.push(value[key]);
        }
        return keys;
    }
}

/**
 * @author Jafar Amini on 2018.
 */
class EnumHandle2 {
    getAsValueTitleTList(myEnum) {
        let value = EnumHandle.getAsObject(myEnum);
        let keys = [];
        for (let key in value) {
            // keys.push({value: key, title: value[key]});
            keys.push(new PairValueTitleT(value[key], key));
        }
        return keys;
    }
    getAsKeyValueTList(myEnum, EnumMap) {
        const value = EnumHandle.getAsObject(myEnum);
        const keyValues = [];
        for (const key in value) {
            if (EnumMap.get(key)) {
                keyValues.push(new PairKeyValueT(key, EnumMap.get(key)));
            }
        }
        return keyValues;
    }
}

let Toolkit2Service = class Toolkit2Service {
    constructor() {
        this.toolkit2 = Toolkit2;
    }
};
Toolkit2Service.ngInjectableDef = ɵɵdefineInjectable({ factory: function Toolkit2Service_Factory() { return new Toolkit2Service(); }, token: Toolkit2Service, providedIn: "root" });
Toolkit2Service = __decorate([
    Injectable({
        providedIn: 'root'
    })
], Toolkit2Service);

/**
 * Async modal dialog service
 * DialogService makes this app easier to test by faking this service.
 * TODO: better modal implementation that doesn't use window.confirm
 */
let DialogService = class DialogService {
    /**
     * Ask user to confirm an action. `message` explains the action and choices.
     * Returns observable resolving to `true`=confirm or `false`=cancel
     */
    confirm(message) {
        const confirmation = window.confirm(message || 'Is it OK?');
        return of(confirmation);
    }
};
DialogService.ngInjectableDef = ɵɵdefineInjectable({ factory: function DialogService_Factory() { return new DialogService(); }, token: DialogService, providedIn: "root" });
DialogService = __decorate([
    Injectable({
        providedIn: 'root'
    })
], DialogService);

/**
 * @author Jafar Amini in March 2018.
 */
var UtilService2;
(function (UtilService2) {
    UtilService2.DialogService = DialogService;
})(UtilService2 || (UtilService2 = {}));

class ListHolder {
    constructor() {
        this._list = [];
    }
    push(item) {
        this._list.push(item);
    }
    remove(filterFunction) {
        this._list = this._list.filter(filterFunction);
    }
    find(filterFunction) {
        return this._list.filter(filterFunction);
    }
    get list() {
        return this._list;
    }
    setList(val) {
        this._list = val;
    }
}

class ListDataService {
    constructor() {
        this.listHolder = new ListHolder();
        this.holderBs = new BehaviorSubject(null);
        this.currentList = this.holderBs.asObservable();
    }
    trigger() {
        this.holderBs.next(this.listHolder.list);
    }
    set(value) {
        this.listHolder.setList(value);
        this.trigger();
    }
    getList() {
        return this.listHolder.list;
    }
    push(item) {
        this.listHolder.push(item);
        this.trigger();
    }
    update(item, getCondition) {
        for (let i = 0; i < this.listHolder.list.length; i++) {
            if (getCondition(this.listHolder.list[i])) {
                this.listHolder.list[i] = clone(item);
                break;
            }
        }
        this.trigger();
    }
    remove(filterFunction) {
        this.listHolder.remove(filterFunction);
        this.trigger();
    }
    find(filterFunction) {
        return this.listHolder.find(filterFunction);
    }
}

class ObjectDataService {
    constructor() {
        this.model = new BehaviorSubject(null);
        this.currentModel = this.model.asObservable();
    }
    changeModel(value) {
        this.model.next(value);
    }
}

/**
 * @author Jafar Amini in December 2018.
 */
class DefaultNotify {
    static notifySuccess(message, title, notifyConfig) {
        const nc = new NotifyConfig(Notify.Type.SUCCESS, Notify.Placement.TOP_LEFT, Notify.TEMPLATES.Template2, message, title);
        if (!isNullOrUndefined(notifyConfig)) {
            ObjectUtil.overlayValues(notifyConfig, nc);
        }
        Notify.showNotify(nc);
    }
    static notifyDanger(message, title, notifyConfig) {
        const nc = new NotifyConfig(Notify.Type.DANGER, Notify.Placement.TOP_LEFT, Notify.TEMPLATES.Template2, message, title);
        if (!isNullOrUndefined(notifyConfig)) {
            ObjectUtil.overlayValues(notifyConfig, nc);
        }
        Notify.showNotify(nc);
    }
    static notifyInfo(message, title, notifyConfig) {
        const nc = new NotifyConfig(Notify.Type.INFO, Notify.Placement.TOP_LEFT, Notify.TEMPLATES.Template2, message, title);
        if (!isNullOrUndefined(notifyConfig)) {
            ObjectUtil.overlayValues(notifyConfig, nc);
        }
        Notify.showNotify(nc);
    }
    static notifyWarning(message, title, notifyConfig) {
        const nc = new NotifyConfig(Notify.Type.WARNING, Notify.Placement.TOP_LEFT, Notify.TEMPLATES.Template2, message, title);
        if (!isNullOrUndefined(notifyConfig)) {
            ObjectUtil.overlayValues(notifyConfig, nc);
        }
        Notify.showNotify(nc);
    }
}
class DefaultNotify2 {
    static notifySuccess(selector, message, title, notifyConfig) {
        const nc = new NotifyConfig(Notify.Type.SUCCESS, Notify.Placement.TOP_LEFT, Notify.TEMPLATES.Template2, message, title);
        if (!isNullOrUndefined(notifyConfig)) {
            ObjectUtil.overlayValues(notifyConfig, nc);
        }
        Notify.showNotify(nc, selector);
    }
    static notifyDanger(selector, message, title, notifyConfig) {
        const nc = new NotifyConfig(Notify.Type.DANGER, Notify.Placement.TOP_LEFT, Notify.TEMPLATES.Template2, message, title);
        if (!isNullOrUndefined(notifyConfig)) {
            ObjectUtil.overlayValues(notifyConfig, nc);
        }
        Notify.showNotify(nc, selector);
    }
    static notifyInfo(selector, message, title, notifyConfig) {
        const nc = new NotifyConfig(Notify.Type.INFO, Notify.Placement.TOP_LEFT, Notify.TEMPLATES.Template2, message, title);
        if (!isNullOrUndefined(notifyConfig)) {
            ObjectUtil.overlayValues(notifyConfig, nc);
        }
        Notify.showNotify(nc, selector);
    }
    static notifyWarning(selector, message, title, notifyConfig) {
        const nc = new NotifyConfig(Notify.Type.WARNING, Notify.Placement.TOP_LEFT, Notify.TEMPLATES.Template2, message, title);
        if (!isNullOrUndefined(notifyConfig)) {
            ObjectUtil.overlayValues(notifyConfig, nc);
        }
        Notify.showNotify(nc, selector);
    }
}

// export * from './pipekit/index';

class SecondHumanizeObject {
    constructor(init = {}) {
        this.days = init.days || 0;
        this.hours = init.hours || 0;
        this.minutes = init.minutes || 0;
        this.seconds = init.seconds || 0;
    }
    getHumanizeString(mode = 'min') {
        let finalStr = '';
        if (this.days > 0) {
            finalStr = Toolkit2.Common.En2Fa(this.days) + ' ' + 'روز';
            if (!(this.hours === 0 && this.minutes === 0 && this.seconds === 0)) {
                finalStr = finalStr + ' و ';
            }
        }
        if (this.hours > 0) {
            finalStr = finalStr + Toolkit2.Common.En2Fa(this.hours) + ' ' + 'ساعت';
            if (!(this.minutes === 0 && this.seconds === 0)) {
                finalStr = finalStr + ' و ';
            }
        }
        if (this.minutes > 0) {
            finalStr = finalStr + Toolkit2.Common.En2Fa(this.minutes) + ' ' + 'دقیقه';
            if (this.seconds > 0 && mode === 'sec') {
                finalStr = finalStr + ' و ';
                finalStr = finalStr + Toolkit2.Common.En2Fa(this.seconds) + ' ' + 'ثانیه';
            }
        }
        if (this.days === 0 && this.hours === 0 && this.minutes === 0 && this.seconds > 0) {
            finalStr = 'چند لحظه پیش';
        }
        return finalStr;
    }
}

class DevisionResult {
    constructor(divisible, divisor) {
        this.quotient = Math.floor(divisible / divisor);
        this.remainder = divisible % divisor;
    }
}

let SecondHumanizePipe = class SecondHumanizePipe {
    transform(value, mode = 'min') {
        const secondHumanizeObject = new SecondHumanizeObject();
        const daySeconds = 86400;
        const hourSeconds = 3600;
        const minSeconds = 60;
        let tmp;
        tmp = new DevisionResult(value, daySeconds);
        secondHumanizeObject.days = tmp.quotient;
        tmp = new DevisionResult(tmp.remainder, hourSeconds);
        secondHumanizeObject.hours = tmp.quotient;
        tmp = new DevisionResult(tmp.remainder, minSeconds);
        secondHumanizeObject.minutes = tmp.quotient;
        secondHumanizeObject.seconds = tmp.remainder;
        return secondHumanizeObject.getHumanizeString(mode);
    }
};
SecondHumanizePipe = __decorate([
    Pipe({
        name: 'secondHumanize'
    })
], SecondHumanizePipe);

let DateToolsModule = class DateToolsModule {
};
DateToolsModule = __decorate([
    NgModule({
        declarations: [SecondHumanizePipe],
        imports: [
            CommonModule
        ],
        exports: [
            SecondHumanizePipe
        ]
    })
], DateToolsModule);

let SafeHtmlPipe = class SafeHtmlPipe {
    constructor(sanitizer) {
        this.sanitizer = sanitizer;
    }
    transform(html, ...args) {
        return this.sanitizer.bypassSecurityTrustHtml(html);
    }
};
SafeHtmlPipe.ctorParameters = () => [
    { type: DomSanitizer }
];
SafeHtmlPipe = __decorate([
    Pipe({
        name: 'safeHtml'
    })
], SafeHtmlPipe);

let NbPipesModule = class NbPipesModule {
};
NbPipesModule = __decorate([
    NgModule({
        imports: [
            CommonModule,
            PipeMomentModule,
            PipeStringModule,
            DateToolsModule
        ],
        declarations: [NbpipePipe, SafeHtmlPipe],
        exports: [
            PipeMomentModule,
            PipeStringModule,
            NbpipePipe,
            DateToolsModule,
            SafeHtmlPipe
        ]
    })
], NbPipesModule);

let UtilModule = class UtilModule {
};
UtilModule = __decorate([
    NgModule({
        declarations: [UtilComponent],
        imports: [
            NbDirectivesModule,
            NbPipesModule
        ],
        exports: [UtilComponent,
            NbDirectivesModule,
            NbPipesModule]
    })
], UtilModule);

/**
 * @author Jafar Amini in October 2018.
 */
var RestServiceResultPolicy;
(function (RestServiceResultPolicy) {
    RestServiceResultPolicy[RestServiceResultPolicy["NONE"] = 'NONE'] = "NONE";
    RestServiceResultPolicy[RestServiceResultPolicy["RESPONSE_CONTENT"] = 'RESPONSE_CONTENT'] = "RESPONSE_CONTENT";
    RestServiceResultPolicy[RestServiceResultPolicy["CUSTOM"] = 'CUSTOM'] = "CUSTOM";
})(RestServiceResultPolicy || (RestServiceResultPolicy = {}));

/**
 * @author Jafar Amini in October 2018.
 */
var RestServiceErrorPolicy;
(function (RestServiceErrorPolicy) {
    RestServiceErrorPolicy[RestServiceErrorPolicy["NONE"] = 'NONE'] = "NONE";
    RestServiceErrorPolicy[RestServiceErrorPolicy["DEFAULT"] = 'DEFAULT'] = "DEFAULT";
    RestServiceErrorPolicy[RestServiceErrorPolicy["CUSTOM"] = 'CUSTOM'] = "CUSTOM";
})(RestServiceErrorPolicy || (RestServiceErrorPolicy = {}));

const FileType = {
    APPLICATION_BPMN20XML_UTF8: 'data:application/bpmn20-xml;charset=UTF-8',
    TEXT_JSON_UTF8: 'data:text/json;charset=utf-8',
    TEXT_PLAIN_UTF8: 'data:text/plain;charset=utf-8',
    IMAGE_SVG: 'data:image/svg+xml'
};

const MediaType = {
    ALL: '*/*',
    APPLICATION_ATOM_XML: 'application/atom+xml',
    APPLICATION_FORM_URLENCODED: 'application/x-www-form-urlencoded',
    APPLICATION_JSON: 'application/json',
    APPLICATION_JSON_UTF8: 'application/json;charset=UTF-8',
    APPLICATION_OCTET_STREAM: 'application/octet-stream',
    APPLICATION_PDF: 'application/pdf',
    APPLICATION_PROBLEM_JSON: 'application/problem+json',
    APPLICATION_PROBLEM_JSON_UTF8: 'application/problem+json;charset=UTF-8',
    APPLICATION_PROBLEM_XML: 'application/problem+xml',
    APPLICATION_RSS_XML: 'application/rss+xml',
    APPLICATION_STREAM_JSON: 'application/stream+json',
    APPLICATION_XHTML_XML: 'application/xhtml+xml',
    APPLICATION_XML: 'application/xml',
    IMAGE_GIF: 'image/gif',
    IMAGE_JPEG: 'image/jpeg',
    IMAGE_PNG: 'image/png',
    MULTIPART_FORM_DATA: 'multipart/form-data',
    TEXT_EVENT_STREAM: 'text/event-stream',
    TEXT_HTML: 'text/html',
    TEXT_MARKDOWN: 'text/markdown',
    TEXT_PLAIN: 'text/plain',
    TEXT_XML: 'text/xml',
};

const Common$1 = {
    PERSIAN_VIRGUL_SEPARATOR: '، ',
    ENGLISH_COMMA_SEPARATOR: ', '
};

var Constants;
(function (Constants) {
    Constants.FileType = FileType;
    Constants.MediaType = MediaType;
    Constants.Common = Common$1;
})(Constants || (Constants = {}));

class BrowsedFile {
}

var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k;
class Css3Colors {
}
Css3Colors.REDS = (_a = class {
    },
    _a.indianRed = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'IndianRed';
        }
        else
            return 'rgba(205,92,92,' + alpha + ')';
    },
    _a.lightCoral = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'LightCoral';
        }
        else
            return 'rgba(240,128,128,' + alpha + ')';
    },
    _a.salmon = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'Salmon';
        }
        else
            return 'rgba(250,128,114,' + alpha + ')';
    },
    _a.darkSalmon = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'DarkSalmon';
        }
        else
            return 'rgba(233,150,122,' + alpha + ')';
    },
    _a.lightSalmon = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'LightSalmon';
        }
        else
            return 'rgba(255,160,122,' + alpha + ')';
    },
    _a.crimson = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'Crimson';
        }
        else
            return 'rgba(220,20,60,' + alpha + ')';
    },
    _a.red = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'Red';
        }
        else
            return 'rgba(255,0,0,' + alpha + ')';
    },
    _a.fireBrick = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'FireBrick';
        }
        else
            return 'rgba(178,34,34,' + alpha + ')';
    },
    _a.darkRed = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'DarkRed';
        }
        else
            return 'rgba(139,0,0,' + alpha + ')';
    },
    _a);
Css3Colors.PINKS = (_b = class {
    },
    _b.pink = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'Pink';
        }
        else
            return 'rgba(255,192,203,' + alpha + ')';
    },
    _b.lightPink = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'LightPink';
        }
        else
            return 'rgba(255,182,193,' + alpha + ')';
    },
    _b.hotPink = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'HotPink';
        }
        else
            return 'rgba(255,105,180,' + alpha + ')';
    },
    _b.deepPink = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'DeepPink';
        }
        else
            return 'rgba(255,20,147,' + alpha + ')';
    },
    _b.mediumVioletRed = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'MediumVioletRed';
        }
        else
            return 'rgba(199,21,133,' + alpha + ')';
    },
    _b.PaleVioletRed = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'PaleVioletRed';
        }
        else
            return 'rgba(219,112,147,' + alpha + ')';
    },
    _b);
Css3Colors.ORANGES = (_c = class {
    },
    _c.coral = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'Coral';
        }
        else
            return 'rgba(255,127,80,' + alpha + ')';
    },
    _c.tomato = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'Tomato';
        }
        else
            return 'rgba(255,99,71,' + alpha + ')';
    },
    _c.orangeRed = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'OrangeRed';
        }
        else
            return 'rgba(255,69,0,' + alpha + ')';
    },
    _c.darkOrange = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'DarkOrange';
        }
        else
            return 'rgba(255,140,0,' + alpha + ')';
    },
    _c.orange = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'Orange';
        }
        else
            return 'rgba(255,165,0,' + alpha + ')';
    },
    _c);
Css3Colors.YELLOWS = (_d = class {
    },
    _d.gold = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'Gold';
        }
        else
            return 'rgba(255,215,0,' + alpha + ')';
    },
    _d.yellow = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'Yellow';
        }
        else
            return 'rgba(255,255,0,' + alpha + ')';
    },
    _d.lightYellow = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'LightYellow';
        }
        else
            return 'rgba(255,255,224,' + alpha + ')';
    },
    _d.lemonChiffon = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'LemonChiffon';
        }
        else
            return 'rgba(255,250,205,' + alpha + ')';
    },
    _d.lightGoldenrodYellow = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'LightGoldenrodYellow';
        }
        else
            return 'rgba(250,250,210,' + alpha + ')';
    },
    _d.papayaWhip = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'PapayaWhip';
        }
        else
            return 'rgba(255,239,213,' + alpha + ')';
    },
    _d.moccasin = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'Moccasin';
        }
        else
            return 'rgba(255,228,181,' + alpha + ')';
    },
    _d.peachPuff = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'PeachPuff';
        }
        else
            return 'rgba(255,218,185,' + alpha + ')';
    },
    _d.paleGoldenrod = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'PaleGoldenrod';
        }
        else
            return 'rgba(238,232,170,' + alpha + ')';
    },
    _d.khaki = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'Khaki';
        }
        else
            return 'rgba(240,230,140,' + alpha + ')';
    },
    _d.darkKhaki = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'DarkKhaki';
        }
        else
            return 'rgba(189,183,107,' + alpha + ')';
    },
    _d);
Css3Colors.Purples = (_e = class {
    },
    _e.lavender = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'Lavender';
        }
        else
            return 'rgba(230,230,250,' + alpha + ')';
    },
    _e.thistle = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'Thistle';
        }
        else
            return 'rgba(216,191,216,' + alpha + ')';
    },
    _e.plum = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'Plum';
        }
        else
            return 'rgba(221,160,221,' + alpha + ')';
    },
    _e.violet = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'Violet';
        }
        else
            return 'rgba(238,130,238,' + alpha + ')';
    },
    _e.orchid = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'Orchid';
        }
        else
            return 'rgba(218,112,214,' + alpha + ')';
    },
    _e.fuchsia = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'Fuchsia';
        }
        else
            return 'rgba(255,0,255,' + alpha + ')';
    },
    _e.magenta = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'Magenta';
        }
        else
            return 'rgba(255,0,255,' + alpha + ')';
    },
    _e.mediumOrchid = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'MediumOrchid';
        }
        else
            return 'rgba(186,85,211,' + alpha + ')';
    },
    _e.mediumPurple = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'MediumPurple';
        }
        else
            return 'rgba(147,112,219,' + alpha + ')';
    },
    _e.blueViolet = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'BlueViolet';
        }
        else
            return 'rgba(138,43,226,' + alpha + ')';
    },
    _e.darkViolet = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'DarkViolet';
        }
        else
            return 'rgba(148,0,211,' + alpha + ')';
    },
    _e.darkOrchid = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'DarkOrchid';
        }
        else
            return 'rgba(153,50,204,' + alpha + ')';
    },
    _e.darkMagenta = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'DarkMagenta';
        }
        else
            return 'rgba(139,0,139,' + alpha + ')';
    },
    _e.purple = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'Purple';
        }
        else
            return 'rgba(128,0,128,' + alpha + ')';
    },
    _e.rebeccaPurple = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'RebeccaPurple';
        }
        else
            return 'rgba(102,51,153,' + alpha + ')';
    },
    _e.indigo = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'Indigo';
        }
        else
            return 'rgba(75,0,130,' + alpha + ')';
    },
    _e.mediumSlateBlue = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'MediumSlateBlue';
        }
        else
            return 'rgba(123,104,238,' + alpha + ')';
    },
    _e.slateBlue = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'SlateBlue';
        }
        else
            return 'rgba(106,90,205,' + alpha + ')';
    },
    _e.darkSlateBlue = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'DarkSlateBlue';
        }
        else
            return 'rgba(72,61,139,' + alpha + ')';
    },
    _e);
Css3Colors.GREENS = (_f = class {
    },
    _f.greenYellow = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'GreenYellow';
        }
        else
            return 'rgba(173,255,47,' + alpha + ')';
    },
    _f.chartreuse = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'Chartreuse';
        }
        else
            return 'rgba(127,255,0,' + alpha + ')';
    },
    _f.lawnGreen = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'LawnGreen';
        }
        else
            return 'rgba(124,252,0,' + alpha + ')';
    },
    _f.lime = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'Lime';
        }
        else
            return 'rgba(0,255,0,' + alpha + ')';
    },
    _f.limeGreen = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'LimeGreen';
        }
        else
            return 'rgba(50,205,50,' + alpha + ')';
    },
    _f.paleGreen = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'PaleGreen';
        }
        else
            return 'rgba(152,251,152,' + alpha + ')';
    },
    _f.lightGreen = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'LightGreen';
        }
        else
            return 'rgba(144,238,144,' + alpha + ')';
    },
    _f.mediumSpringGreen = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'MediumSpringGreen';
        }
        else
            return 'rgba(0,250,154,' + alpha + ')';
    },
    _f.springGreen = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'SpringGreen';
        }
        else
            return 'rgba(0,255,127,' + alpha + ')';
    },
    _f.mediumSeaGreen = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'MediumSeaGreen';
        }
        else
            return 'rgba(60,179,113,' + alpha + ')';
    },
    _f.seaGreen = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'SeaGreen';
        }
        else
            return 'rgba(46,139,87,' + alpha + ')';
    },
    _f.forestGreen = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'ForestGreen';
        }
        else
            return 'rgba(34,139,34,' + alpha + ')';
    },
    _f.green = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'Green';
        }
        else
            return 'rgba(0,128,0,' + alpha + ')';
    },
    _f.darkGreen = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'DarkGreen';
        }
        else
            return 'rgba(0,100,0,' + alpha + ')';
    },
    _f.yellowGreen = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'YellowGreen';
        }
        else
            return 'rgba(154,205,50,' + alpha + ')';
    },
    _f.oliveDrab = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'OliveDrab';
        }
        else
            return 'rgba(107,142,35,' + alpha + ')';
    },
    _f.olive = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'Olive';
        }
        else
            return 'rgba(128,128,0,' + alpha + ')';
    },
    _f.darkOliveGreen = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'DarkOliveGreen';
        }
        else
            return 'rgba(85,107,47,' + alpha + ')';
    },
    _f.mediumAquamarine = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'MediumAquamarine';
        }
        else
            return 'rgba(102,205,170,' + alpha + ')';
    },
    _f.darkSeaGreen = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'DarkSeaGreen';
        }
        else
            return 'rgba(143,188,143,' + alpha + ')';
    },
    _f.lightSeaGreen = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'LightSeaGreen';
        }
        else
            return 'rgba(32,178,170,' + alpha + ')';
    },
    _f.darkCyan = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'DarkCyan';
        }
        else
            return 'rgba(0,139,139,' + alpha + ')';
    },
    _f.teal = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'Teal';
        }
        else
            return 'rgba(0,128,128,' + alpha + ')';
    },
    _f);
Css3Colors.BLUES_CYANS = (_g = class {
    },
    _g.aqua = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'Aqua';
        }
        else
            return 'rgba(0,255,255,' + alpha + ')';
    },
    _g.cyan = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'Cyan';
        }
        else
            return 'rgba(0,255,255,' + alpha + ')';
    },
    _g.lightCyan = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'LightCyan';
        }
        else
            return 'rgba(224,255,255,' + alpha + ')';
    },
    _g.paleTurquoise = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'PaleTurquoise';
        }
        else
            return 'rgba(175,238,238,' + alpha + ')';
    },
    _g.aquamarine = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'Aquamarine';
        }
        else
            return 'rgba(127,255,212,' + alpha + ')';
    },
    _g.turquoise = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'Turquoise';
        }
        else
            return 'rgba(64,224,208,' + alpha + ')';
    },
    _g.mediumTurquoise = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'MediumTurquoise';
        }
        else
            return 'rgba(72,209,204,' + alpha + ')';
    },
    _g.darkTurquoise = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'DarkTurquoise';
        }
        else
            return 'rgba(0,206,209,' + alpha + ')';
    },
    _g.cadetBlue = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'CadetBlue';
        }
        else
            return 'rgba(95,158,160,' + alpha + ')';
    },
    _g.steelBlue = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'SteelBlue';
        }
        else
            return 'rgba(70,130,180,' + alpha + ')';
    },
    _g.lightSteelBlue = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'LightSteelBlue';
        }
        else
            return 'rgba(176,196,222,' + alpha + ')';
    },
    _g.powderBlue = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'PowderBlue';
        }
        else
            return 'rgba(176,224,230,' + alpha + ')';
    },
    _g.lightBlue = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'LightBlue';
        }
        else
            return 'rgba(173,216,230,' + alpha + ')';
    },
    _g.skyBlue = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'SkyBlue';
        }
        else
            return 'rgba(135,206,235,' + alpha + ')';
    },
    _g.lightSkyBlue = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'LightSkyBlue';
        }
        else
            return 'rgba(135,206,250,' + alpha + ')';
    },
    _g.deepSkyBlue = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'DeepSkyBlue';
        }
        else
            return 'rgba(0,191,255,' + alpha + ')';
    },
    _g.dodgerBlue = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'DodgerBlue';
        }
        else
            return 'rgba(30,144,255,' + alpha + ')';
    },
    _g.cornflowerBlue = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'CornflowerBlue';
        }
        else
            return 'rgba(100,149,237,' + alpha + ')';
    },
    _g.royalBlue = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'RoyalBlue';
        }
        else
            return 'rgba(65,105,225,' + alpha + ')';
    },
    _g.blue = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'Blue';
        }
        else
            return 'rgba(0,0,255,' + alpha + ')';
    },
    _g.mediumBlue = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'MediumBlue';
        }
        else
            return 'rgba(0,0,205,' + alpha + ')';
    },
    _g.darkBlue = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'DarkBlue';
        }
        else
            return 'rgba(0,0,139,' + alpha + ')';
    },
    _g.navy = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'Navy';
        }
        else
            return 'rgba(0,0,128,' + alpha + ')';
    },
    _g.midnightBlue = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'MidnightBlue';
        }
        else
            return 'rgba(25,25,112,' + alpha + ')';
    },
    _g);
Css3Colors.BROWNS = (_h = class {
    },
    _h.cornsilk = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'Cornsilk';
        }
        else
            return 'rgba(255,248,220,' + alpha + ')';
    },
    _h.blanchedAlmond = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'BlanchedAlmond';
        }
        else
            return 'rgba(255,235,205,' + alpha + ')';
    },
    _h.bisque = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'Bisque';
        }
        else
            return 'rgba(255,228,196,' + alpha + ')';
    },
    _h.navajoWhite = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'NavajoWhite';
        }
        else
            return 'rgba(255,222,173,' + alpha + ')';
    },
    _h.wheat = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'Wheat';
        }
        else
            return 'rgba(245,222,179,' + alpha + ')';
    },
    _h.burlyWood = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'BurlyWood';
        }
        else
            return 'rgba(222,184,135,' + alpha + ')';
    },
    _h.tan = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'Tan';
        }
        else
            return 'rgba(210,180,140,' + alpha + ')';
    },
    _h.rosyBrown = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'RosyBrown';
        }
        else
            return 'rgba(188,143,143,' + alpha + ')';
    },
    _h.sandyBrown = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'SandyBrown';
        }
        else
            return 'rgba(244,164,96,' + alpha + ')';
    },
    _h.goldenrod = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'Goldenrod';
        }
        else
            return 'rgba(218,165,32,' + alpha + ')';
    },
    _h.darkGoldenrod = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'DarkGoldenrod';
        }
        else
            return 'rgba(184,134,11,' + alpha + ')';
    },
    _h.peru = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'Peru';
        }
        else
            return 'rgba(205,133,63,' + alpha + ')';
    },
    _h.chocolate = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'Chocolate';
        }
        else
            return 'rgba(210,105,30,' + alpha + ')';
    },
    _h.saddleBrown = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'SaddleBrown';
        }
        else
            return 'rgba(139,69,19,' + alpha + ')';
    },
    _h.sienna = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'Sienna';
        }
        else
            return 'rgba(160,82,45,' + alpha + ')';
    },
    _h.brown = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'Brown';
        }
        else
            return 'rgba(165,42,42,' + alpha + ')';
    },
    _h.maroon = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'Maroon';
        }
        else
            return 'rgba(128,0,0,' + alpha + ')';
    },
    _h);
Css3Colors.WHITES = (_j = class {
    },
    _j.white = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'White';
        }
        else
            return 'rgba(255,255,255,' + alpha + ')';
    },
    _j.snow = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'Snow';
        }
        else
            return 'rgba(255,250,250,' + alpha + ')';
    },
    _j.honeydew = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'Honeydew';
        }
        else
            return 'rgba(240,255,240,' + alpha + ')';
    },
    _j.mintCream = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'MintCream';
        }
        else
            return 'rgba(245,255,250,' + alpha + ')';
    },
    _j.azure = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'Azure';
        }
        else
            return 'rgba(240,255,255,' + alpha + ')';
    },
    _j.aliceBlue = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'AliceBlue';
        }
        else
            return 'rgba(240,248,255,' + alpha + ')';
    },
    _j.ghostWhite = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'GhostWhite';
        }
        else
            return 'rgba(248,248,255,' + alpha + ')';
    },
    _j.whiteSmoke = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'WhiteSmoke';
        }
        else
            return 'rgba(245,245,245,' + alpha + ')';
    },
    _j.seashell = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'Seashell';
        }
        else
            return 'rgba(255,245,238,' + alpha + ')';
    },
    _j.beige = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'Beige';
        }
        else
            return 'rgba(245,245,220,' + alpha + ')';
    },
    _j.oldLace = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'OldLace';
        }
        else
            return 'rgba(253,245,230,' + alpha + ')';
    },
    _j.floralWhite = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'FloralWhite';
        }
        else
            return 'rgba(255,250,240,' + alpha + ')';
    },
    _j.ivory = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'Ivory';
        }
        else
            return 'rgba(255,255,240,' + alpha + ')';
    },
    _j.antiqueWhite = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'AntiqueWhite';
        }
        else
            return 'rgba(250,235,215,' + alpha + ')';
    },
    _j.linen = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'Linen';
        }
        else
            return 'rgba(250,240,230,' + alpha + ')';
    },
    _j.lavenderBlush = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'LavenderBlush';
        }
        else
            return 'rgba(255,240,245,' + alpha + ')';
    },
    _j.mistyRose = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'MistyRose';
        }
        else
            return 'rgba(255,228,225,' + alpha + ')';
    },
    _j);
Css3Colors.GRAYS = (_k = class {
    },
    _k.gainsboro = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'Gainsboro';
        }
        else
            return 'rgba(220,220,220,' + alpha + ')';
    },
    _k.lightGray = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'LightGray';
        }
        else
            return 'rgba(211,211,211,' + alpha + ')';
    },
    // public static lightGrey = (alpha?: number) => {
    //   if (isNullOrUndefined(alpha)) {
    //     return 'LightGrey';
    //   }
    //   else return 'rgba(211,211,211,' + alpha + ')';
    // };
    _k.silver = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'Silver';
        }
        else
            return 'rgba(192,192,192,' + alpha + ')';
    },
    _k.darkGray = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'DarkGray';
        }
        else
            return 'rgba(169,169,169,' + alpha + ')';
    },
    // public static darkGrey = (alpha?: number) => {
    //   if (isNullOrUndefined(alpha)) {
    //     return 'DarkGrey';
    //   }
    //   else return 'rgba(169,169,169,' + alpha + ')';
    // };
    _k.gray = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'Gray';
        }
        else
            return 'rgba(128,128,128,' + alpha + ')';
    },
    // public static grey = (alpha?: number) => {
    //   if (isNullOrUndefined(alpha)) {
    //     return 'Grey';
    //   }
    //   else return 'rgba(128,128,128,' + alpha + ')';
    // };
    _k.dimGray = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'DimGray';
        }
        else
            return 'rgba(105,105,105,' + alpha + ')';
    },
    // public static dimGrey = (alpha?: number) => {
    //   if (isNullOrUndefined(alpha)) {
    //     return 'DimGrey';
    //   }
    //   else return 'rgba(105,105,105,' + alpha + ')';
    // };
    _k.lightSlateGray = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'LightSlateGray';
        }
        else
            return 'rgba(119,136,153,' + alpha + ')';
    },
    // public static lightSlateGrey = (alpha?: number) => {
    //   if (isNullOrUndefined(alpha)) {
    //     return 'LightSlateGrey';
    //   }
    //   else return 'rgba(119,136,153,' + alpha + ')';
    // };
    _k.slateGray = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'SlateGray';
        }
        else
            return 'rgba(112,128,144,' + alpha + ')';
    },
    // public static slateGrey = (alpha?: number) => {
    //   if (isNullOrUndefined(alpha)) {
    //     return 'SlateGrey';
    //   }
    //   else return 'rgba(112,128,144,' + alpha + ')';
    // };
    _k.darkSlateGray = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'DarkSlateGray';
        }
        else
            return 'rgba(47,79,79,' + alpha + ')';
    },
    // public static darkSlateGrey = (alpha?: number) => {
    //   if (isNullOrUndefined(alpha)) {
    //     return 'DarkSlateGrey';
    //   }
    //   else return 'rgba(47,79,79,' + alpha + ')';
    // };
    _k.black = (alpha) => {
        if (isNullOrUndefined(alpha)) {
            return 'Black';
        }
        else
            return 'rgba(0,0,0,' + alpha + ')';
    },
    _k);

class DeleteConfirmationResponse {
    constructor(init = {}) {
        this.confirmed = init.confirmed;
        this.pickModel = init.pickModel;
    }
}

class PickModel {
    constructor(init = {}) {
        !isUndefined(init.id) ? this.id = init.id : this.id = null;
        !isUndefined(init.index) ? this.index = init.index : this.index = null;
        !isUndefined(init.data) ? this.data = init.data : this.data = null;
    }
}

class DeleteModel {
    constructor() {
        this.message = null;
        this.status = 'not-started';
        this.pickModel = new PickModel();
    }
}

class GeneralDomain {
}

class IdObject {
    constructor(id) {
        this.id = !isUndefined(id) ? id : null;
    }
}

class LabelObject {
    constructor(label) {
        if (!isUndefined(label)) {
            this.label = label;
        }
    }
}

class ListContainer {
    constructor() {
        this.data = new Array();
    }
}

/**
 * @author Jafar Amini in April 2018.
 */
var OrderType;
(function (OrderType) {
    OrderType[OrderType["ASC"] = 'asc'] = "ASC";
    OrderType[OrderType["DESC"] = 'desc'] = "DESC";
})(OrderType || (OrderType = {}));

class PageContainer {
    constructor() {
        this.content = [];
        this.pageable = new Pageable();
        this.totalPages = null;
        this.totalElements = null;
        this.last = null;
        this.numberOfElements = null;
        this.sort = new Sort();
        this.first = null;
        this.size = null;
        this.number = null;
    }
}
class Sort {
    constructor() {
        this.unsorted = null;
        this.sorted = null;
    }
}
class Pageable {
    constructor() {
        this.sort = new Sort();
        this.pageSize = null;
        this.pageNumber = null;
        this.offset = null;
        this.unpaged = null;
        this.paged = null;
    }
}

class PickForDest {
    constructor(data, index, id) {
        this.data = data;
        this.index = index;
        this.id = id;
    }
}

class ForCheck {
    constructor(type) {
        this.checked = false;
        this.checked = false;
        this.data = new type;
    }
}

class JsonQuery {
}

/**
 * @author Jafar Amini in May 2018.
 */
class Paging {
    constructor(page, start, size) {
        // this.page = page || null;
        // this.size = size || null;
        // this.start = start || null;
        if (!isNullOrUndefined(page)) {
            this.page = page;
        }
        else {
            this.page = null;
        }
        if (!isNullOrUndefined(start)) {
            this.start = start;
        }
        else {
            this.start = null;
        }
        if (!isNullOrUndefined(size)) {
            this.size = size;
        }
        else {
            this.size = null;
        }
        // this.page = page ? page : null;
        // this.start = start ? start : null;
        // this.size = size ? size : null;
    }
    static getSample() {
        return new Paging(1, 1, 1);
    }
}

class Sorting {
    // for which the default value and the allowed
    // values are different per query implementation.
    constructor(order, sort) {
        this.order = order;
        this.sort = sort;
    }
}

class SpringSorting {
}

/**
 * @author Jafar Amini in May 2018.
 */
class SearchCommonQuery {
    constructor(init = {}) {
        !isUndefined(init.term) ? this.term = init.term : init.term = null;
        !isUndefined(init.totalElements) ? this.totalElements = init.totalElements : init.totalElements = null;
        !isUndefined(init.paging) ? this.paging = init.paging : init.paging = new Paging();
        !isUndefined(init.sortings) ? this.sortings = init.sortings : init.sortings = [];
        !isUndefined(init.options) ? this.options = init.options : init.options = null;
    }
}

class ListHelper {
    constructor(init) {
        this.paging = init.paging || new Paging();
        this.sortings = init.sortings || [];
        this.term = init.term || '';
        this.options = init.options || {};
    }
}

var ModalSize;
(function (ModalSize) {
    ModalSize["DEFAULT"] = "DEFAULT";
    ModalSize["LARGE"] = "LARGE";
    ModalSize["SMALL"] = "SMALL";
    ModalSize["X_LARGE"] = "X_LARGE";
    ModalSize["FULL_SCREEN"] = "FULL_SCREEN";
    ModalSize["FULL_WIDTH"] = "FULL_WIDTH";
    ModalSize["CUSTOM_WIDTH"] = "CUSTOM_WIDTH";
})(ModalSize || (ModalSize = {}));

/**
 * @author Jafar Amini
 */
var ChartDefault = {
    label: '',
    labels: [],
    data: [],
    colors: null,
    chartType: null,
    legend: true,
    options: {
        scaleShowVerticalLines: false,
        responsive: true,
        legend: {
            labels: {
                fontColor: 'black',
                fontFamily: null
                // fontFamily: 'IRANSans-web'
            },
        },
        scales: {
            yAxes: [{
                    ticks: {
                        beginAtZero: true
                    }
                }]
        }
    }
};

/**
 * @author Jafar Amini
 */
class Chart {
    constructor(init = {}) {
        this.options = {
            scaleShowVerticalLines: null,
            responsive: null,
            legend: {
                labels: {
                    // This more specific font property overrides the global property
                    fontColor: null,
                    fontFamily: null
                }
            },
            scales: {
                yAxes: [{
                        ticks: {
                            beginAtZero: null
                        }
                    }]
            }
        };
        this.label = init.label || ChartDefault.label;
        this.labels = init.labels || ChartDefault.labels;
        this.data = init.data || ChartDefault.data;
        this.colors = init.colors || ChartDefault.colors;
        this.chartType = init.chartType || ChartDefault.chartType;
        this.legend = init.legend || ChartDefault.legend;
        if (isNullOrUndefined(init.options)) {
            this.options = ChartDefault.options;
        }
        else {
            this.options.scaleShowVerticalLines = init.options.scaleShowVerticalLines || ChartDefault.options.scaleShowVerticalLines;
            if (isNullOrUndefined(init.options.legend)) {
                this.options.legend = ChartDefault.options.legend;
            }
            else {
                if (isNullOrUndefined(init.options.legend.labels)) {
                    this.options.legend.labels = ChartDefault.options.legend.labels;
                }
                else {
                    this.options.legend.labels.fontColor =
                        init.options.legend.labels.fontColor || ChartDefault.options.legend.labels.fontColor;
                    this.options.legend.labels.fontFamily =
                        init.options.legend.labels.fontFamily || ChartDefault.options.legend.labels.fontFamily;
                }
            }
            if (isNullOrUndefined(init.options.scales)) {
                this.options.scales = ChartDefault.options.scales;
            }
            else {
                this.options.scales.yAxes = ChartDefault.options.scales.yAxes;
            }
        }
    }
    static quickCreate(options = {}) {
        const chart = new Chart();
        if (!isNullOrUndefined(options.beginAtZero)) {
            chart.options.scales.yAxes = [{ ticks: { beginAtZero: options.beginAtZero } }];
        }
        if (!isNullOrUndefined(options.fontColor)) {
            chart.options.legend.labels.fontColor = options.fontColor;
        }
        if (!isNullOrUndefined(options.fontFamily)) {
            chart.options.legend.labels.fontFamily = options.fontFamily;
        }
        return chart;
    }
}

/**
 * @author Jafar Amini
 */
var ChartType;
(function (ChartType) {
    ChartType[ChartType["pie"] = 'pie'] = "pie";
    ChartType[ChartType["line"] = 'line'] = "line";
    ChartType[ChartType["bar"] = 'bar'] = "bar";
    ChartType[ChartType["radar"] = 'radar'] = "radar";
    ChartType[ChartType["polarArea"] = 'polarArea'] = "polarArea";
    ChartType[ChartType["doughnut"] = 'doughnut'] = "doughnut";
})(ChartType || (ChartType = {}));

/**
 * @author Jafar Amini
 */
class BarChart extends Chart {
    constructor(init = {}) {
        super();
        const myChart = new Chart(init);
        myChart.chartType = ChartType.bar;
        return myChart;
    }
}

/**
 * @author Jafar Amini
 */
class DoughnutChart extends Chart {
    constructor(init = {}) {
        super();
        const myChart = new Chart(init);
        myChart.chartType = ChartType.doughnut;
        return myChart;
    }
}

/**
 * @author Jafar Amini
 */
class LineChart extends Chart {
    constructor(init = {}) {
        super();
        const myChart = new Chart(init);
        myChart.chartType = ChartType.line;
        return myChart;
    }
}

/**
 * @author Jafar Amini
 */
class PieChart extends Chart {
    constructor(init = {}) {
        super();
        const myChart = new Chart(init);
        myChart.chartType = ChartType.pie;
        return myChart;
    }
}

/**
 * @author Jafar Amini
 */
class PolarAreaChart extends Chart {
    constructor(init = {}) {
        super();
        const myChart = new Chart(init);
        myChart.chartType = ChartType.polarArea;
        return myChart;
    }
}

/**
 * @author Jafar Amini
 */
class RadarChart extends Chart {
    constructor(init = {}) {
        super();
        const myChart = new Chart(init);
        myChart.chartType = ChartType.radar;
        return myChart;
    }
}

/**
 * @author Jafar Amini
 */
class ChartExtra {
    constructor(type) {
        // private type;
        this.show = false;
        // this.type = type;
        this.show = false;
        this.query = new type;
    }
}

/**
 // Usage in module
 export function HttpLoaderFactory(http: HttpClient) {
  let reqHeaders = new HttpHeaders();
  reqHeaders = reqHeaders.set('Authorization',
    localStorage.getItem('token_type') + ' ' + localStorage.getItem('access_token'));
  return new CustomTranslateHttpLoader(http, 'http://localhost:8081/salam', {key: 'a'}, reqHeaders);
}
 @NgModule({
 ... ,
 imports: [
 ... ,
 TranslateModule.forRoot({
      loader: {
        provide: TranslateLoader,
        useFactory: HttpLoaderFactory,
        deps: [HttpClient]
      }
    }),
 ...
 ],
 ...
 })
 */
class CustomTranslateHttpLoader {
    constructor(http, url, query, reqHeaders) {
        this.http = http;
        this.url = url;
        this.query = query;
        this.reqHeaders = reqHeaders;
    }
    /**
     * Gets the translations from the server
     */
    getTranslation(lang, remote = false) {
        if (remote === true) {
            return this.getRemoteJson(lang);
        }
        else {
            return this.getLocalJson(lang);
        }
    }
    getLocalJson(lang) {
        return this.http.get(`${this.url}/${lang}.json`);
    }
    getRemoteJson(lang) {
        return this.http.get(this.url + '?locale=' + lang + '&' +
            Toolkit2.StringUtil.objectToQueryString2(this.query), { headers: this.reqHeaders });
    }
}

/*
    Languages codes are ISO_639-1 codes, see http://en.wikipedia.org/wiki/List_of_ISO_639-1_codes
    They are written in English to avoid character encoding issues (not a perfect solution)
*/
const languages = {
    'en': { rtl: false, name: 'English' },
    'fa': { rtl: true, name: 'فارسی' },
    'ar': { rtl: true, name: 'العربية' },
    'tr': { rtl: false, name: 'Türkçe' },
    'az': { rtl: false, name: 'Azərbaycan dili' },
    'hy': { rtl: false, name: 'Հայերեն' },
    'ca': { rtl: false, name: 'Català' },
    'zh-cn': { rtl: false, name: '中文（简体）' },
    'zh-tw': { rtl: false, name: '繁體中文' },
    'cs': { rtl: false, name: 'Český' },
    'da': { rtl: false, name: 'Dansk' },
    'nl': { rtl: false, name: 'Nederlands' },
    'et': { rtl: false, name: 'Eesti' },
    'fr': { rtl: false, name: 'Français' },
    'gl': { rtl: false, name: 'Galego' },
    'de': { rtl: false, name: 'Deutsch' },
    'el': { rtl: false, name: 'Ελληνικά' },
    'hi': { rtl: false, name: 'हिंदी' },
    'hu': { rtl: false, name: 'Magyar' },
    'id': { rtl: false, name: 'Bahasa Indonesia' },
    'it': { rtl: false, name: 'Italiano' },
    'ja': { rtl: false, name: '日本語' },
    'ko': { rtl: false, name: '한국어' },
    'mr': { rtl: false, name: 'मराठी' },
    'pl': { rtl: false, name: 'Polski' },
    'pt-br': { rtl: false, name: 'Português (Brasil)' },
    'pt-pt': { rtl: false, name: 'Português' },
    'ro': { rtl: false, name: 'Română' },
    'ru': { rtl: false, name: 'Русский' },
    'sk': { rtl: false, name: 'Slovenský' },
    'sr': { rtl: false, name: 'Srpski' },
    'es': { rtl: false, name: 'Español' },
    'sv': { rtl: false, name: 'Svenska' },
    'ta': { rtl: false, name: 'தமிழ்' },
    'th': { rtl: false, name: 'ไทย' },
    'ua': { rtl: false, name: 'Українська' },
    'vi': { rtl: false, name: 'Tiếng Việt' }
};

let FindLanguageFromKeyPipe = class FindLanguageFromKeyPipe {
    transform(lang) {
        return languages[lang].name;
        // return this.languages[lang].name;
    }
    isRTL(lang) {
        return languages[lang].rtl;
    }
};
FindLanguageFromKeyPipe.ngInjectableDef = ɵɵdefineInjectable({ factory: function FindLanguageFromKeyPipe_Factory() { return new FindLanguageFromKeyPipe(); }, token: FindLanguageFromKeyPipe, providedIn: "root" });
FindLanguageFromKeyPipe = __decorate([
    Pipe({ name: 'findLanguageFromKey' }),
    Injectable({ providedIn: 'root' })
], FindLanguageFromKeyPipe);

/**
 // Usage in module
 export function HttpLoaderFactory(http: HttpClient) {
  let reqHeaders = new HttpHeaders();
  reqHeaders = reqHeaders.set('Authorization',
    localStorage.getItem('token_type') + ' ' + localStorage.getItem('access_token'));
  return new CustomTranslateHttpLoader(http, 'http://localhost:8081/salam', {key: 'a'}, reqHeaders);
}
 @NgModule({
 ... ,
 imports: [
 ... ,
 TranslateModule.forRoot({
      loader: {
        provide: TranslateLoader,
        useFactory: HttpLoaderFactory,
        deps: [HttpClient]
      }
    }),
 ...
 ],
 ...
 })
 */
class JoinedCustomTranslateHttpLoader {
    constructor(http, url, query, reqHeaders) {
        this.http = http;
        this.url = url;
        this.query = query;
        this.reqHeaders = reqHeaders;
    }
    /**
     * Gets the translations from the server
     */
    getTranslation(lang, remote = false) {
        if (remote === true) {
            return this.getRemoteJson(lang);
        }
        else {
            this.getLocalJson(lang).subscribe((res) => {
                console.log('JOINEDDDDDDDDDDDDDDDDD', res);
            });
            return this.getLocalJson(lang);
        }
        // return this.http.get(`${this.prefix}${lang}${this.suffix}`);
    }
    getLocalJson(lang) {
        // return this.http.get(`${this.url[1]}/${lang}.json`);
        return forkJoin(this.http.get(`${this.url[0]}/${lang}.json`).pipe(take(1)), this.http.get(`${this.url[1]}/${lang}.json`).pipe(take(1))).pipe(map(([f1Json, f2Json]) => {
            return Object.assign({}, f1Json, f2Json); // Uses object spreed
            // Object.keys(f1Json)
            // // Now we can map the keys to the actual user objects
            // // and merge them with the homework
            //   .map(userKey => {
            //     const user = users[userKey];
            //     const homework = homeworkMap[userKey];
            //     return {
            //       ...user,
            //       homework
            //     };
            //   });
        }));
    }
    getRemoteJson(lang) {
        return forkJoin(this.http.get(this.url[0] + '?lang=' + lang + '&' +
            Toolkit2.StringUtil.objectToQueryString2(this.query), { headers: this.reqHeaders }), this.http.get(this.url[1] + '?lang=' + lang + '&' +
            Toolkit2.StringUtil.objectToQueryString2(this.query), { headers: this.reqHeaders }));
    }
}

let LanguageHelper = class LanguageHelper {
    constructor(translateService, 
    // tslint:disable-next-line: no-unused-variable
    rootRenderer, findLanguageFromKeyPipe, titleService, router) {
        this.translateService = translateService;
        this.rootRenderer = rootRenderer;
        this.findLanguageFromKeyPipe = findLanguageFromKeyPipe;
        this.titleService = titleService;
        this.router = router;
        this.renderer = null;
        this.renderer = rootRenderer.createRenderer(document.querySelector('html'), null);
        this.init();
    }
    // switchLanguage(languageKey: string) {
    //   this.translateService.use(languageKey);
    //   // this.accessToLabelsFromTypescript();
    // }
    //
    // setDefaultLang(lang) {
    //   this.translateService.setDefaultLang(lang);
    // }
    //
    // // /**
    // //  * Sample that shows we can access to labels value in current language
    // //  */
    // // accessToLabelsFromTypescript() {
    // //   const intro = this.translateService.instant('Intro');
    // //   console.log(intro);
    // // }
    setLanguageList(languageCodes, set) {
        this.getAll(languageCodes).then((langs) => {
            set(langs);
        });
    }
    getAll(LANGUAGES) {
        return Promise.resolve(LANGUAGES);
    }
    /**
     * Update the window title using params in the following
     * order:
     * 1. titleKey parameter
     * 2. $state.$current.data.pageTitle (current state page title)
     * 3. 'global.title'
     */
    updateTitle(titleKey) {
        if (!titleKey) {
            titleKey = this.getPageTitle(this.router.routerState.snapshot.root);
        }
        this.translateService.get(titleKey).subscribe((title) => {
            this.titleService.setTitle(title);
        });
    }
    init() {
        this.translateService.onLangChange.subscribe((event) => {
            this.renderer.setAttribute(document.querySelector('html'), 'lang', this.translateService.currentLang);
            this.updateTitle();
            this.updatePageDirection();
        });
    }
    getPageTitle(routeSnapshot) {
        let title = (routeSnapshot.data && routeSnapshot.data.pageTitle) ?
            routeSnapshot.data.pageTitle : 'myapp5App';
        if (routeSnapshot.firstChild) {
            title = this.getPageTitle(routeSnapshot.firstChild) || title;
        }
        return title;
    }
    updatePageDirection() {
        this.renderer.setAttribute(document.querySelector('html'), 'dir', this.findLanguageFromKeyPipe.isRTL(this.translateService.currentLang) ? 'rtl' : 'ltr');
    }
};
LanguageHelper.ctorParameters = () => [
    { type: TranslateService },
    { type: RendererFactory2 },
    { type: FindLanguageFromKeyPipe },
    { type: Title },
    { type: Router }
];
LanguageHelper.ngInjectableDef = ɵɵdefineInjectable({ factory: function LanguageHelper_Factory() { return new LanguageHelper(ɵɵinject(TranslateService), ɵɵinject(RendererFactory2), ɵɵinject(FindLanguageFromKeyPipe), ɵɵinject(Title), ɵɵinject(Router)); }, token: LanguageHelper, providedIn: "root" });
LanguageHelper = __decorate([
    Injectable({ providedIn: 'root' })
], LanguageHelper);

let NbLanguageModule = class NbLanguageModule {
};
NbLanguageModule = __decorate([
    NgModule({
        declarations: [FindLanguageFromKeyPipe],
        imports: [
            CommonModule
        ],
        exports: [
            FindLanguageFromKeyPipe
        ]
    })
], NbLanguageModule);

// export * from './nb-lang-change-event';
// export * from './nb-translate-service';

// export interface IRestRequestOptions {
//   // resetStatusWaitTime?: number;
// }
class RestRequestOptions {
    constructor() {
        // constructor(arg: IRestRequestOptions = {}) {
        this.status = 'not-started';
        // this.resetStatusWaitTime = !isNullOrUndefined(arg.resetStatusWaitTime) ? arg.resetStatusWaitTime : 500;
    }
}

/*
 * Public API Surface of util
 */

/**
 * Generated bundle index. Do not edit.
 */

export { ActionMode, BarChart, BrowsedFile, CategorizedPairKeyValue, CategorizedPairNameValue, CategorizedPairValueTitle, Chart, ChartExtra, ChartType, CommonDevTools, Constants, Css3Colors, CustomTranslateHttpLoader, DefaultNotify, DefaultNotify2, DeleteConfirmationResponse, DeleteModel, DevisionResult, DialogService, DoughnutChart, ElementType, EnumHandle, EnumHandle2, FindLanguageFromKeyPipe, ForCheck, GeneralDomain, IdObject, Interaction, JoinedCustomTranslateHttpLoader, JsonQuery, LabelObject, LanguageHelper, LineChart, ListContainer, ListDataService, ListHelper, ListHolder, ModalSize, NbDirectivesModule, NbLanguageModule, Notify, NotifyConfig, NotifyConfigOptions, NotifyConfigSettings, NotifyPlacement, NotifyType, ObjectDataService, ObjectUtilT, OrderDirection, OrderType, PageContainer, Pageable, Paging, PairIdTitle, PairKeyTitle, PairKeyValue, PairKeyValueT, PairNameValue, PairValueTitle, PairValueTitleT, PickForDest, PickModel, PieChart, PolarAreaChart, QueryOperatorEnum, RadarChart, ReadMode, RequestStatus, RestRequestOptions, RestServiceErrorPolicy, RestServiceResultPolicy, SearchCommonQuery, Sort, Sorting, SpringSorting, Toolkit2, Toolkit2Service, UnFlatifyOptions, UtilComponent, UtilModule, UtilService, UtilService2, clone, copyTextToClipboard, isBlankString, isFunction, isNullOrUndefined, isNullOrUndefinedOrBlankString, isObject, isUndefined, languages, objAndSameType, AreYouSureDirective as ɵa, NbPipesModule as ɵb, PipeMomentModule as ɵc, IsoToJDayPipe as ɵd, IsoToJMinPipe as ɵe, IsoToJSecPipe as ɵf, IsoToJalaliPipe as ɵg, PipeStringModule as ɵh, ShortenPipe as ɵi, DateToolsModule as ɵj, SecondHumanizePipe as ɵk, NbpipePipe as ɵl, SafeHtmlPipe as ɵm, TEMPLATES as ɵn };
//# sourceMappingURL=angular-boot-util.js.map
